# -*- coding: utf-8 -*-

require 'date'
require 'sidfm_controller_support'

class SidfmProductsController < ApplicationController
  unloadable
  
  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private

  include SidfmControllerSupport
  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data
 
  before_action :check_roles
 
  before_action :get_vm_version
  before_action :clear_import_session
  before_action :clear_export_session
  before_action -> { get_summary_table_lines('products') }

  def check_roles
    if !@sidfm_roles.include?(:use) then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
  end

  def create_content 
    product_ids =[]
    version_ids = []
    lproduct_ids = []
    lversion_ids = []
    product_hosts = {}
    version_hosts = {}
    lproduct_hosts = {}
    lversion_hosts = {}
    is_mng = !@sidfm_roles.include?(:admin)
    user_cats = User.current.categories
    SidfmdHostApp.joins(:sidfmd_host).each do |app|
      #報告者権限は自分のカテゴリに属するホストのプロダクトのみ
      if is_mng then 
        host_cats = app.sidfmd_host.sidfmd_categories.where(:is_system => false)
        if ((host_cats - user_cats).size == host_cats.size) then
          next
        end
      end
      host_name = { 
        :hostname => app.sidfmd_host.hostname,
        :display_name => (app.sidfmd_host.ext_name ? app.sidfmd_host.ext_name : app.sidfmd_host.hostname ),
        :ext_name => app.sidfmd_host.ext_name
      }
      ver_id = app.version_id
      if !ver_id || ver_id == 0 then
        prod_id = app.product_id
        product_ids << prod_id
        product_hosts[prod_id] = [] unless product_hosts[prod_id]
        product_hosts[prod_id] << host_name
      else
        version_ids << ver_id
        version_hosts[ver_id] = [] unless version_hosts[ver_id]
        version_hosts[ver_id] << host_name
      end
    end
    SidfmdHostLapp.joins(:sidfmd_host).each do |lapp|
      #報告者権限は自分のカテゴリに属するホストのプロダクトのみ
      if is_mng then
        host_cats = lapp.sidfmd_host.sidfmd_categories.where(:is_system => false)
        if ((host_cats - user_cats).size == host_cats.size) then
          next
        end
      end
      host_name = {
        :hostname => lapp.sidfmd_host.hostname,
        :display_name => (lapp.sidfmd_host.ext_name ? lapp.sidfmd_host.ext_name : lapp.sidfmd_host.hostname),
        :ext_name => lapp.sidfmd_host.ext_name
      }
      ver_id = lapp.sidfmd_lversion_id
      if ver_id == 0 then
        prod_id = lapp.sidfmd_lproduct_id
        lproduct_ids << prod_id
        lproduct_hosts[prod_id] = [] unless lproduct_hosts[prod_id]
        lproduct_hosts[prod_id] << host_name
      else
        lversion_ids << ver_id
        lversion_hosts[ver_id] = [] unless lversion_hosts[ver_id]
        lversion_hosts[ver_id] << host_name
      end
    end
    get_sorted_host_names = ->(host_names){
      host_names.sort{|a, b|
        r = a[:display_name].downcase <=> b[:display_name].downcase
        if r == 0 then
          r = a[:hostname] <=> b[:hostname]
        end
        r
      }
      return host_names
    }
    product_version_table = SidfmdVersion.joins('LEFT OUTER JOIN sidfmd_products ON sidfmd_products.id = sidfmd_versions.sidfmd_product_id')
                                         .select(
                                           :pid,
                                           :vid,
                                           :name,
                                           :fullname,
                                           'sidfmd_versions.is_support_end AS v_is_support_end' 
                                         )
    lproduct_version_table = SidfmdLversion.joins('FULL OUTER JOIN sidfmd_lproducts ON sidfmd_lproducts.id = sidfmd_lversions.sidfmd_lproduct_id')
                                           .select(
                                             'sidfmd_lproducts.id AS lpid',
                                             'sidfmd_lversions.id AS lvid',
                                             :name,
                                             :fullname
                                           )
    all_products = SidfmdProduct.where('pid IN (?)', product_ids).map{|p|
                     { 
                       :pid => p.pid, 
                       :name => p.name,
                       :version_name => '全て', 
                       :is_support_end => p.is_support_end,
                       :hosts => get_sorted_host_names.(product_hosts[p.pid]),
                       :product_id => p.pid
                     }
                   }.concat(
                     product_version_table.where(:vid => version_ids).map{|v|
                       {
                         :vid => v.vid,
                         :name => v.name,
                         :version_name => v.fullname,
                         :is_support_end => v.v_is_support_end,
                         :hosts => get_sorted_host_names.(version_hosts[v.vid]),
                         :product_id => v.pid
                       }
                     }
                   ).concat(
                     SidfmdLproduct.where('id IN (?)', lproduct_ids).map{|p|
                       {     
                         :lpid => p.id,     
                         :name => p.name,
                         :version_name => '全て',     
                         :is_support_end => true,
                         :hosts => get_sorted_host_names.(lproduct_hosts[p.id]),
                         :lproduct_id => p.id
                       }
                     }
                   ).concat(
                     lproduct_version_table.where('sidfmd_lversions.id IN (?)', lversion_ids).map{|v|
                       {
                         :lvid => v.lvid,
                         :name => v.name,
                         :version_name => v.fullname,
                         :is_support_end => true,
                         :hosts => get_sorted_host_names.(lversion_hosts[v.lvid]),
                         :lproduct_id => v.lpid
                       }
                     }
                   )

  end

  FILTER_KEYS = {}

  #フィルターの初期化
  def init_filter_keys
    if FILTER_KEYS.size == 0 then
      FILTER_KEYS[:search_products] = ''
      FILTER_KEYS[:search_versions] = ''
      FILTER_KEYS[:search_hosts] = ''
    end
  end

  def check_filters
    init_filter_keys
    FILTER_KEYS.each do |key, val|
      unless @products[:filters].has_key?(key) then
        @products[:filters][key] = val
      end
    end
    @default_filters = FILTER_KEYS
  end
 
  def filter_products
    searched_prod_name = @products[:search_products]
    searched_prod_version = @products[:search_versions]
    searched_prod_host_name = @products[:search_hosts]
    if searched_prod_name.length > 0 then
      searched_prod_name = searched_prod_name.downcase
      @all_products = @all_products.reject{|p| p[:name].downcase.index(searched_prod_name) == nil}
    end
    if searched_prod_version.length > 0 then
      searched_prod_version = searched_prod_version.downcase
      @all_products = @all_products.select{|p|(p[:version_name].downcase.index(searched_prod_version) != nil)}
    end
    if searched_prod_host_name.length > 0 then
      searched_prod_host_name = searched_prod_host_name.downcase
      @all_products = @all_products.select{|p|
        p[:hosts].select{|h| 
          (h[:ext_name] && h[:ext_name].downcase.index(searched_prod_host_name) != nil) || (h[:hostname].downcase.index(searched_prod_host_name) != nil) 
        }.size > 0
      }
    end
  end

  def sort_products 
    #order_param = @products[:order].split('_')
    last_index = @products[:order].rindex('_')
    col = @products[:order].slice(0..last_index - 1)#order_param[0]
    order = @products[:order].slice(last_index + 1..@products[:order].size - 1)#order_param[1]
    if col == 'product-name' then
      @all_products = @all_products.sort{|a,b| 
        r = a[:name].downcase <=> b[:name].downcase
        if r == 0 then
          r = (a[:version_name] == '全て' ? 100 : 1000) <=> (b[:version_name] == '全て' ? 100 :1000)
          if r == 0 then
            r = a[:version_name] <=> b[:version_name]
          end
        end
        r
      }
      @all_products = @all_products.reverse if order == "desc"
    elsif col == 'product-is-support-end' then
      if order == "asc" then
        @all_products = @all_products.sort{|a, b|
          r = (a[:is_support_end] ? 1000 : 100) <=> (b[:is_support_end] ? 1000 : 100)
          if r == 0 then
            r = a[:name].downcase <=> b[:name].downcase
            if r == 0 then
              r = (a[:version_name] == '全て' ? 100 : 1000) <=> (b[:version_name] == '全て' ? 100 :1000)
              if r == 0 then
                r = a[:version_name] <=> b[:version_name]
              end
            end
          end
          r
        }
      else
        @all_products = @all_products.sort{|b, a|
          r = (a[:is_support_end] ? 1000 : 100) <=> (b[:is_support_end] ? 1000 : 100)
          if r == 0 then
            r = b[:name].downcase <=> a[:name].downcase
            if r == 0 then
              r = (a[:version_name] == '全て' ? 1000 : 100) <=> (b[:version_name] == '全て' ? 1000 :100)
              if r == 0 then
                r = a[:version_name] <=> b[:version_name]
              end
            end
          end
          r
        }
      end
    elsif col == 'product-version-name' then
      version_all_products = []
      version_not_all_products = []
      @all_products.each do |p|
        if p[:version_name] == '全て' then
          version_all_products << p
        else
          version_not_all_products << p
        end
      end
      if order == "asc" then
        @all_products = version_all_products.sort_by{|p| p[:name]} + 
                        version_not_all_products.sort{|a, b|
                          r = a[:version_name] <=> b[:version_name]
                          if r == 0 then
                            r = (a[:is_support_end] ? 1000 : 100) <=> (b[:is_support_end] ? 1000 : 100)
                          end
                          r
                        }
=begin
        @all_products = @all_products.sort{|a, b|
          r = (a[:version_name] == '全て' ? 100 : 1000) <=> (b[:version_name] == '全て' ? 100 :1000)
          if r == 0 then
            r = a[:version_name] <=> b[:version_name]
            if r == 0 then
              r = (a[:is_support_end] ? 1000 : 100) <=> (b[:is_support_end] ? 1000 : 100)
            end
          end
          r
        }
=end
      else
        @all_products = version_not_all_products.sort{|b, a|
                          r = a[:version_name] <=> b[:version_name]
                          if r == 0 then
                            r = (a[:is_support_end] ? 1000 : 100) <=> (b[:is_support_end] ? 1000 : 100)
                          end
                          r
                        } + version_all_products.sort_by{|p| p[:name]}
=begin
        @all_products = @all_products.sort{|b, a|
          r = (a[:version_name] == '全て' ? 100 : 1000) <=> (b[:version_name] == '全て' ? 100 : 1000)
          if r == 0 then
            r = a[:version_name] <=> b[:version_name]
            if r == 0 then
              r = (a[:is_support_end] ? 1000 : 100) <=> (b[:is_support_end] ? 1000 : 100)
            end
          end
          r
        }
=end
      end
    else #どれにも当てはまらない場合は、プロダクト名でsort(普通の操作では起きない)
      @all_products = @all_products.sort{|a, b|
        r = a[:name].downcase <=> b[:name].downcase
        if r == 0 then
          r = (a[:version_name] == '全て' ? 100 : 1000) <=> (b[:version_name] == '全て' ? 100 :1000)
          if r == 0 then
            r = a[:version_name] <=> b[:version_name]
          end
        end
        r
      }
    end
  end

  def create_product_params(prod)
    prod_info = 
      if prod[:pid] then
        { 'pid' => prod[:pid] }
      elsif prod[:vid] then
        { 'vid' => prod[:vid] }
      elsif prod[:lpid] then
        { 'lpid' => prod[:lpid] }
      else
        { 'lvid' => prod[:lvid] }
      end
  end

  def parse_params
    f_params = params.fetch(:pages, {}).permit(:offset, :per_page, :order, :filters, :checked, :all_checked)
    @products = {}
    @products[:per_page] = (@summary_table_lines || f_params[:per_page] || 10).to_i
    @products[:offset] = (f_params[:offset] || 0).to_i
    @products[:filters] = JSON.parse(f_params[:filters] || '{}').symbolize_keys
    @products[:order] = (f_params[:order] || 'product-name_asc')
    @products[:checked] = JSON.parse(f_params[:checked] || '[]')
    @products[:all_checked] = ((f_params[:all_checked] || '') == 'true')
    @products[:search_products] = @products[:filters][:search_products] || ''
    @products[:search_versions] = @products[:filters][:search_versions] || ''
    @products[:search_hosts] = @products[:filters][:search_hosts] || ''
    check_filters
  end

  def create_summary_product_html(prod, **opts)
    prod_params = create_product_params(prod)
    data = {
      :chkbox => view_context.create_summary_product_chkbox(prod_params, opts[:checked]),
      'product-name' => view_context.create_summary_product_name(prod),
      'product-version-name' => view_context.create_summary_product_version_name(prod[:version_name]),
      :hosts => view_context.create_summary_product_hosts(prod[:hosts]),
      'product-is-support-end' => view_context.create_summary_product_is_support_end(prod[:is_support_end]),
      :link_to_pages => view_context.create_summary_product_link_to_pages(prod)
    }
  end
 
  public 

  def index
    parse_params
    #@all_products = create_content
    #filter_products
    #sort_products
    #@products[:total_size] = @all_products.size
    #@products[:content] = @all_products.slice(@products[:offset].. (@products[:offset] + @products[:per_page] - 1))
    #フィルター後のid一覧
    #@all_product_ids = @all_products.map{|prod|
    #  create_product_params(prod)
    #}

  end

  CSV_BOM = "\xff\xfe".force_encoding('UTF-16LE').freeze

  def download
    f_params = params.fetch(:download, {}).permit(:format, :order, :filters, :product_ids)
    format = f_params[:format]
    if !['json','tsv'].include?(format) then 
      redirect_to :action => :index , :notice => '不正な入力です。' and return
    end
    begin 
      product_ids = JSON.parse(f_params[:product_ids] || '[]')
    rescue JSON::ParserError => e
      logger.error(e)
    end
    @products = {}
    @products[:filters] = JSON.parse(f_params[:filters] || '{}').symbolize_keys
    @products[:order] = (f_params[:order] || 'product-name_asc')
    @products[:search_products] = @products[:filters][:search_products] || ''
    @products[:search_versions] = @products[:filters][:search_versions] || ''
    @products[:search_hosts] = @products[:filters][:search_hosts] || ''
    @all_products = create_content
    filter_products
    if product_ids.size > 0 then
      pids = []
      vids = []
      lpids = []
      lvids = []
      product_ids.each do |id_data|
        id_data = id_data.symbolize_keys
        if id_data[:vid] then
          vids << id_data[:vid] 
        elsif id_data[:pid] then
          pids << id_data[:pid]
        elsif id_data[:lvid] then
          lvids << id_data[:lvid]
        elsif id_data[:lpid]
          lpids << id_data[:lpid]
        end
      end
      @all_products = @all_products.select{|p|
        (p[:vid] && vids.include?(p[:vid])) ||
        (p[:pid] && pids.include?(p[:pid])) ||
        (p[:lvid] && lvids.include?(p[:lvid])) ||
        (p[:lpid] && lpids.include?(p[:lpid]))
      }    
    end
    sort_products
    if format == 'json' then
      json = {}
      order_param = @products[:order].split('_')
      if order_param[0] == 'product-is-support-end'
        order_param[0] = 'is_sidfm_support'
      end
      json[:order] = { :by => order_param[0], :type => order_param[1] }
      json[:filters] = @products[:filters]
      json[:content] = []
      setting_hostname = Setting.host_name
      setting_protocol = Setting.protocol
      @all_products.each do |prod_data|
        pid = prod_data[:pid]
        vid = prod_data[:vid]
        lpid = prod_data[:lpid]
        lvid = prod_data[:lvid]
        item = {}
        if pid then
          prod_info = { 'pid' => pid }
          item[:product_id] = pid
        elsif vid then
          prod_info = { 'vid' => vid }
          item[:version_id] = vid
        elsif lpid then
          prod_info = { 'lpid' => lpid }
          item[:lproduct_id] = lpid
        else
          prod_info = { 'lvid' => lvid }
          item[:lversion_id] = lvid
        end
        link_to_hosts = url_for(
          :controller => :sidfm_hosts,
          :action => :index_r,
          :'pages[filters]' => { 'product' => prod_info }.to_json,
        )
        link_to_vul = url_for(
          :controller => :sidfm_vulnerabilities,
          :action => :index_r,
          :'pages[filters]' => { 'product' => prod_info }.to_json,
        )
        item[:product_name] = prod_data[:name]
        item[:version_name] = prod_data[:version_name]
        item[:hosts] = prod_data[:hosts]
        item[:is_sidfm_support] = !prod_data[:is_support_end]
        item[:link_to_hosts_page] = link_to_hosts
        item[:link_to_vulnerabilities_page] = link_to_vul
        json[:content] << item
      end 
      send_data(json.to_json, :filename => "vmlist-products-tsv-##{Time.current.strftime('%Y%m%d%H%M%S')}.json", :disposition => 'attachment')
      return
    elsif format == 'tsv' then
      header = [ 'プロダクト名', 'バージョン名', 'ホスト名', 'SIDfmサポート', 'ホスト別状況画面へのリンク', '脆弱性別状況画面へのリンク' ].map{|c| c.encode('UTF-16LE')}
      csv = CSV.generate("#{CSV_BOM}".force_encoding('UTF-16LE'), :col_sep => "\t".encode('UTF-16LE')) do |o|
        o << header
        item = []
        @all_products.each do |prod_data|
          item.clear
          pid = prod_data[:pid]
          vid = prod_data[:vid]
          lpid = prod_data[:lpod]
          lvid = prod_data[:lvid]
          if pid then
            prod_info = { 'pid' => pid }
          elsif vid then
            prod_info = { 'vid' => vid }
          elsif lpid then
            prod_info = { 'lpid' => lpid }
          else
            prod_info = { 'lvid' => lvid }
          end
          link_to_hosts = url_for(
            :controller => 'sidfm_hosts',
            :action => 'index_r',
            :'pages[filters]' => { 'product' => prod_info }.to_json 
          )
          link_to_vul = url_for(
            :controller => :sidfm_vulnerabilities,
            :action => :index_r,
            :'pages[filters]' => { 'product' => prod_info }.to_json
          )
          item << prod_data[:name].encode('UTF-16LE')
          item << prod_data[:version_name].encode('UTF-16LE')
          item << prod_data[:hosts].map{|h| h[:hostname]}.join(',').encode('UTF-16LE')
          item << (prod_data[:is_support_end] ? '無' : '有').encode('UTF-16LE')
          item << link_to_hosts.encode('UTF-16LE')
          item << link_to_vul.encode('UTF-16LE')
          o << item   
        end
      end
      send_data(csv, :filename => "vmlist-products-tsv#{Time.current.strftime('%Y%m%d%H%M%S')}.csv", :disposition => 'attachment')
      return   
    end

    redirect_to :action => :index, :notice => 'ファイルの出力に失敗しました。'
  end

  def api_get_display_summary_content
    GC.start
    f_params = params.fetch(:pages, {}).permit(:per_page)
    parse_params
    @products[:per_page] = f_params[:per_page].to_i
    @all_products = create_content
    filter_products
    sort_products
    rows = @all_products.slice(@products[:offset].. (@products[:offset] + @products[:per_page] - 1)) || []
    ret = { 
      :rows => rows.map{|prod| create_summary_product_html(prod, { :checked =>  @products[:checked] })},
      :total => @all_products.count, 
      :all_ids => @all_products.map{|prod| create_product_params(prod)},
      :sidfm_roles => @sidfm_roles,
      :user_cats => (@sidfm_roles.include?(:admin) ? nil : User.current.categories.pluck(:id))
    }
    render :layout => false, :json => ret
  end

end
