#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmLicensesHelper

  # サマリー表のチェックボックス
  def create_summary_license_chkbox(host, checked)
    content = check_box_tag(
      'table[selected]',
      host[:hostname],
      checked.include?(host[:hostname]),
      :class => "#{host[:flg_delete] ? "" : "hostSelect"}",
      :id => "#{host[:flg_delete] ? "" : "table_selected_#{host[:id].to_s}"}",
      :data => { :deleted => host[:flg_delete] } #削除状態が分かるようにするための要素。jsでclassを付与するために使用。
    )
  end

  # サマリー表のゴミ箱
  def create_summary_license_del_flag(flg_delete)
    content = flg_delete ? content_tag(:i, nil, :class => "far fa-trash-alt c_gray_40") : ""
  end

  # サマリー表の名称変更ボタン
  def create_summary_license_rename_button(host)
    button_tag(
      "名称変更",
      :type => "button",
      :name => "button",
      :class => "change-name-btn btn btn-default btn-xs text-nowrap mb-1",
      :role => "button",
      :autocomplete => "off",
      :data => {
        :hostname => host[:hostname],
        :hostid => "#{host[:id]}",
        :displayname => "#{host[:sidfmd_host] ? (host[:sidfmd_host][:ext_name] ? host[:sidfmd_host][:ext_name] : "") : ""}",
        :value => "#{host[:sidfmd_host] ? host[:sidfmd_host].id : ""}"
      }
    )
  end

  # サマリー表の同期状態
  def create_summary_license_sync_status(host, last_received_at, sidfm_roles)
    last_received_at = last_received_at ? last_received_at.strftime('%Y/%m/%d %H:%M:%S') : '---'
    content =
      if host[:sidfmd_host] then
        if host[:sidfmd_host].last_checked_at then
          if host[:sidfmd_host].is_syncing then
            content_tag(
              :div,
              :data => {
                :toggle => "tooltip",
                :html => "true",
                :title => "データ受信日時: #{last_received_at}<br/>脆弱性更新日時: #{host[:sidfmd_host].get_display_last_checked_at}"
              }
            ) do
              concat(
                button_tag(
                  "同期",
                  :class => "#{sidfm_roles.include?(:use) ? "synchronize-status-btn " : ""}btn btn-success btn-xs ml-2 mb-1 text-nowrap",
                  :data => { :value => "#{host[:sidfmd_host].id}" },
                  :style => "opacity:0.9;"
                )
              )
            end
          else
            content_tag(
              :div,
              :data => {
                :toggle => "tooltip",
                :html => "true",
                :title => "データ受信日時: #{last_received_at}<br/>脆弱性更新日時: #{host[:sidfmd_host].get_display_last_checked_at}"
              }
            ) do
              concat(
                button_tag(
                  "非同期",
                  :class => "#{sidfm_roles.include?(:use) ? "synchronize-status-btn " : ""}btn btn-warning btn-xs ml-2 mb-1 text-nowrap",
                  :data => { :value => "#{host[:sidfmd_host].id}" },
                  :style => "opacity:0.9;"
                )
              )
            end
          end
        else
          if host[:sidfmd_host].is_agent then
            content_tag(
              :div,
              :data => {
                :toggle => "tooltip",
                :html => "true",
                :title => "データ受信日時: #{last_received_at}<br/>同期待機中"
              }
            ) do
              concat(
                button_tag(
                  "非同期",
                  :class => "#{sidfm_roles.include?(:use) ? "synchronize-status-btn " : ""}btn btn-warning btn-xs ml-2 mb-1 text-nowrap",
                  :data => { :value => "#{host[:sidfmd_host].id}" },
                  :style => "opacity:0.9;"
                )
              )
            end
          else
            content_tag(
              :div,
              :data => { :toggle => "tooltip", :html => "true", :title => "同期待機中" }
            ) do
              concat(
                button_tag(
                  "非同期",
                  :class => "#{sidfm_roles.include?(:use) ? "synchronize-status-btn " : ""}btn btn-warning btn-xs ml-2 mb-1 text-nowrap",
                  :data => { :value => "#{host[:sidfmd_host].id}" },
                  :style => "opacity:0.9;"
                )
              )
            end
          end
        end
      else
        button_tag(
          "非同期",
          :type => "button",
          :class => "btn btn-warning btn-xs ml-2 mb-1 text-nowrap",
          :style => "opacity:0.9;",
          :data => { :toggle => "tooltip", :title => "登録に問題があるため同期無効" }
        )
      end
  end

  # サマリー表のホスト名
  def create_summary_license_hostname(host, last_received_at, sidfm_roles)
    #ホスト名が長い時、ホスト名のspanにtolltipを指定するとtooltipの位置がおかしくなるので、親のdivにtooltipを指定する
    content = host[:sidfmd_host] ?
      content_tag(
        :div,
        :class => "text-truncate pl-1 d-inline-block max-width-100",
        :data => {
          :toggle => "tooltip",
          :html => "true",
          :placement => "right",
          :title => "ホスト名: #{host[:hostname]}<br/>表示名: #{host[:sidfmd_host][:ext_name] ? host[:sidfmd_host][:ext_name] : "(未設定)"}#{host[:sidfmd_host].memo ? "<br/>メモ: #{host[:sidfmd_host].memo}" : ""}"
        }
      ) do
        concat(
          link_to(
            { :controller => :sidfm_host, :action => :index, :id => host[:sidfmd_host].id },
            :class => "g-text-ul"
          ) do
            concat(
              content_tag(
                :span,
                "#{host[:sidfmd_host][:ext_name] ? host[:sidfmd_host][:ext_name] : host[:hostname]}",
                :class => "align-middle"
              )
            )
          end
        )
      end :
      content_tag(       
        :div,
        :class => "text-truncate pl-1 d-inline-block max-width-100",
        :data => {
          :toggle => "tooltip",
          :html => "true",
          :placement => "right",
          :title => "ホスト名: #{host[:hostname]}<br/>表示名: (未設定)"
        }
      ) do
        concat( content_tag(:span, host[:hostname], :class => "align-middle") )
      end
    content = content.concat(
      content_tag(
        :div,
        :class => "d-flex"
      ) do
        concat( (sidfm_roles.include?(:use) or sidfm_roles.include?(:admin)) ? 
          create_summary_license_rename_button(host) :
          ""
        )
        .concat( create_summary_license_sync_status(host, last_received_at, sidfm_roles) )
      end
    )
  end
 
  # サマリー表のエージェントフラグ
  def create_summary_license_agent_flag(is_agent)
    content = is_agent ? content_tag(:i, nil, :class => "fas fa-circle c_gray_30") : ""
  end

  # サマリー表の手動登録ホストフラグ
  def create_summary_license_manual_flag(is_manual)
    content = is_manual ? content_tag(:i, nil, :class => "fas fa-circle c_gray_30") : ""
  end

  # サマリー表のカテゴリ
  def create_summary_license_categories(host, system_category, custom_categories, sidfm_roles, prior_category)
    content =
      if host[:sidfmd_host] then
        ret = ''
        #s_cat = host[:sidfmd_host].system_category
        #ns_cats = SidfmdCategory.where(:id =>
        #  host[:sidfmd_host].no_system_categories(sidfm_roles.include?(:admin) ? nil : user).select(:sidfmd_category_id)
        #).order('LOWER(name COLLATE "C")')
        #ns_cats = host[:sidfmd_host].no_system_categories(sidfm_roles.include?(:admin) ? nil : user)
        #                           .joins(:sidfmd_category)
        #                          .order('LOWER(sidfmd_categories.name COLLATE "C")')
        #                         .select('sidfmd_categories.name')
        if system_category then
          if custom_categories.size == 0 then
            ret = content_tag(:div) do
              concat(
                content_tag(
                  :span,
                  :class => "align-middle g-font-size-11",
                  :data => { :toggle => "tooltip", :placement => "right", :title => "#{system_category[:desc]}" },
                ) do
                  concat(
                    link_to(
                      system_category[:name],
                      '#',
                      :class => 'ignore-click category-change',
                      :data => { :categoryid => system_category[:id], :filteritem => 'category_id', :name => 'category_id', :value => system_category[:id] }
                    )
                  )
                end
              )
            end
          else
            ret = content_tag(:div) do
              concat(
                create_summary_host_categories(
                  host[:sidfmd_host][:id],
                  system_category[:id],
                  system_category[:name],
                  custom_categories,
                  prior_category,
                 opts = { :button_class => "btn btn-xs btn-light-nb2 text-nowrap g-width-9em my-1" }
                )
              )
            end
            #ret = create_categories_balloon([ s_cat ], ns_cats, prior_category, { :filteritem => 'category_id', :name => 'category_id' })
          end
        else
          ret = content_tag(
            :span,
            :class => "align-middle mb-1 g-width-9em text-red g-font-size-11 font-weight-bold"
          ) do
            concat(
              "カテゴリ未登録"
            )
            .concat(
              content_tag(
                :span,
                :data => {
                  :toggle => "tooltip",
                  :title => "カテゴリが登録されておりません。カテゴリを登録するまで大半の脆弱性マッチングの集計から除外されます。"
                }
              ) do
                concat(
                  content_tag(:i, nil, :class => "fas fa-question-circle text-red", :style => "margin-left: 1px;" )
                )
              end
            )
          end
        end
        if (sidfm_roles & [ :host_admin ]).size > 0 then
          #all_cats = ([ s_cat ] + ns_cats).compact
          ret = ret.concat(
            button_tag(
              "カテゴリ変更",
              :type => "submit",
              :class => "btn btn-default btn-xs text-nowrap mb-1",
              :role => "button",
              :autocomplete => "off",
              :data => {
                #:toggle => "modal",
                #:target => "#categoriesModal",
                :modify => "host_categories",
                :sidfmdhostid => "#{host[:sidfmd_host][:id]}",
                :hostname => "#{host[:hostname]}",
                #:syscategory => "#{s_cat ? s_cat.to_json : ""}",
                #:categories => "#{all_cats.to_json}",
                :title => "カテゴリ編集 - #{host[:sidfmd_host][:ext_name] ? host[:sidfmd_host][:ext_name] : host[:hostname]}"
              }
            )
          )
        end
        ret
      else
        content_tag(
          :span,
          :class => "align-middle mb-1 g-width-8em text-red g-font-size-11 font-weight-bold"
        ) do
          concat(
            "関連付け無し"
          )
          .concat(
            content_tag(
              :span,
              :data => {
                :toggle => "tooltip",
                :title => "ライセンスでは使用されていますが、本VMサーバには登録されていない状態です。同じホスト名でホストを登録する、ホストを削除してライセンスを開放する、のいずれかを行うことを推奨します。"
              }
            ) do
              concat( content_tag(:i, nil, :class => "fas fa-question-circle text-red", :style => "margin-left: 1px;") )
            end
          )
        end
      end
  end

  # サマリー表のOS名
  def create_summary_license_osname(osname)
    #content = ''
    #if host[:sidfmd_host] then
    #  os = host[:sidfmd_host].get_os
    #  if os then
    #    content = os.has_key?(:osname) ? content_tag(:div, os[:osname]) : '---'
    #  else
    #    content = "---"
    #  end
    #else
    #  content = "---"
    #end
    content = osname ? osname : '---'
  end

  # サマリー表の検出アプリ
  def create_summary_license_apps(app_count, host_id)
    content = content_tag(
      :div
    ) do
      concat(
        if app_count > 0 then
          content_tag(
            :div,
            :class => "balloon-dropdown"
          ) do
            concat(
              button_tag(
                :type => "button",
                :class => "btn btn-xs btn-light-nb2 text-nowrap action-open-list create-balloon dropdown-without-arrow text-nowrap dropdown-toggle caret-off",
                :data => { :appsize => app_count, :showlist => "apps", :hostid => host_id },
              ) do
                concat(
                  content_tag(
                    :span,
                    :class => "copy-tooltip-btn",
                    :data => { :toggle => "tooltip", :placement => "bottom", :title => "コピーしました", :trigger => "manual" }
                  ) do
                    concat(
                      content_tag(:span, "検出アプリ")
                    )
                    .concat(
                      content_tag(
                        :span,
                        "#{app_count}",
                        :class => "badge badge-pill badge-gray-999", 
                        :style => "margin-left: 1px;"
                      )
                    )
                  end
                )
              end
            )
            .concat(
              content_tag(
                :div,
                :class => "dropdown-menu invisible",
                :style => "width: 300px;",
                :data => { :listmenu => true }
              ) do
                concat(
                  content_tag(
                    :div,
                    :class => "balloon-menu",
                    :style => "padding-bottom:0px !important; max-width:330px !important;"
                  ) do
                    concat(
                      content_tag(:div, nil, :class => "balloon-arrow")
                    )
                    .concat(
                      content_tag(
                        :div,
                        :class => "dropdown-item bg-white",
                        :style => "overflow-x: hidden;"
                      ) do
                        concat(
                          content_tag(
                            :div,
                            nil,
                            :id => "app-list#{host_id}",
                            :class => "list-group"
                          )
                        )
                      end
                    )
                    .concat(
                      content_tag(
                        :div,
                        :class => "dropdown-item bg-white",
                        :style => "margin-top: 1px;"
                      ) do
                        content_tag(
                          :div,
                          :class => "float-right mb-1"
                        ) do
                          concat(
                            button_tag(
                              "コピー",
                              :id => "copy-app-list#{host_id}",
                              :class => "btn btn-primary btn-xs g-font-size-0_74x copybtn text-nowrap",
                              #"data-clipboard-target" => "#app-list#{host_id}"
                            )
                          )
                        end
                      end
                    )
                  end
                )
              end
            )
          end
        else
          "---"
        end
      )
    end
  end

  # サマリー表のパッケージ変更ボタン
  def create_summary_license_change_pkgs_button(host, pkg_count, os)
    content = content_tag(
      :div,
      :class => "position-relative"
    ) do
      concat(           
        button_tag(
          "パッケージ変更",
          :type => 'button',
          :class => 'btn btn-default btn-xs text-nowrap modify-pkgs-btn mt-1',
          :data => {
            :hostid => host[:sidfmd_host].id,
            :displayname => host[:sidfmd_host].display_name,
            :os => os,
            :agent => host[:sidfmd_host].is_agent,
            :pkgsize => pkg_count 
          }
        )
      )
      .concat(
        content_tag(
          :div,
          :class => "spinner-border text-secondary pkg-loading-mark",
          :id => "pkg-loading-mark-#{host[:sidfmd_host].id}",
          :role => "status",
          :style => "display: none;"
        ) do
          concat(
            content_tag(:span, "Loading...", :class => "sr-only")
          )
        end
      )
    end
  end

  # サマリー表のパッケージ登録ボタン
  def create_summary_license_regist_pkgs_button(host, pkg_count, os)
    button_tag(
      "パッケージ登録",
      :type => 'button',
      :class => 'btn btn-default btn-xs text-nowrap modify-pkgs-btn mt-1',
      :data => {
        :hostid => host[:sidfmd_host].id,
        :displayname => host[:sidfmd_host].display_name,
        :pkgsize => pkg_count,
        :os => os,
        :agent => host[:sidfmd_host].is_agent
      }
    )  
  end

  # サマリー表のパッケージ
  def create_summary_license_pkgs(host, pkg_count, os, sidfm_roles)
    #os = nil
    #if host[:sidfmd_host] then
    #  os = host[:sidfmd_host].get_os
    #  if os and os.has_key?(:osname) then
    #    os = os[:osname]
    #  else
    #    os = nil
    #  end
    #end

    host_id = host[:sidfmd_host].id    

    content = content_tag(
      :div,
      :class => "mt-1",
      :data => { :pkgarea => true }
    ) do
      concat(
        if pkg_count > 0 then
          content_tag(
            :div,
            :class => "balloon-dropdown"
          ) do
            concat(
              button_tag(
                :type => "button",
                :class => "btn btn-xs btn-light-nb2 text-nowrap action-open-list create-balloon dropdown-without-arrow text-nowrap dropdown-toggle caret-off",
                :data => { :showlist => "pkgs", :hostid => host_id, :pkgsize => pkg_count },
              ) do
                concat(
                  content_tag(
                    :div,
                    :class => "d-inline copy-tooltip-btn",
                    :data => { :toggle => "tooltip", :placement => "bottom", :title => "コピーしました", :trigger => "manual" }
                  ) do
                    concat(
                      content_tag(:span, "パッケージ")
                    )
                    .concat(
                      content_tag(
                        :span,
                        "#{pkg_count}",
                        :class => "badge badge-pill badge-gray-999",
                        :style => "margin-left: 1px;"
                      )
                    )
                  end
                )
              end
            )
            .concat(
              content_tag(
                :div,
                :class => "dropdown-menu invisible",
                :style => "width: 300px;",
                :data => { :listmenu => true }
              ) do
                concat(
                  content_tag(
                    :div,
                    :class => "balloon-menu",
                    :style => "padding-bottom:0px !important; max-width:330px !important;"
                  ) do
                    concat(
                      content_tag(:div, nil, :class => "balloon-arrow")
                    )
                    .concat(
                      content_tag(
                        :div,
                        :class => "dropdown-item bg-white",
                        :style => "overflow-x: hidden; max-height: 200px;"
                      ) do
                        concat(
                          content_tag(
                            :div,
                            nil,
                            :id => "pkg-list#{host_id}",
                            :class => "list-group"
                          )
                        )
                      end
                    )
                    .concat( 
                      content_tag(
                        :div,
                        :class => "dropdown-item bg-white",
                        :style => "margin-top: 1px;"
                      ) do
                        concat(
                          content_tag(
                            :div,
                            :class => "float-right mb-1"
                          ) do
                            concat(
                              button_tag(
                                "コピー",
                                :id => "copy-pkg-list#{host_id}",
                                :class => "btn btn-primary btn-xs g-font-size-0_74x copybtn text-nowrap",
                                "data-clipboard-target" => "#pkg-list#{host_id}"
                              )
                            )
                          end
                        )
                      end
                    )
                  end
                )
              end
            )
            .concat(
              if sidfm_roles.include?(:host_admin) or sidfm_roles.include?(:host_limited_admin) then
                create_summary_license_change_pkgs_button(host, pkg_count, os)
              else
                ""
              end
            )
          end
        else
          if host[:sidfmd_host] and (sidfm_roles.include?(:host_admin) or sidfm_roles.include?(:host_limited_admin)) then
            create_summary_license_regist_pkgs_button(host, pkg_count, os)
          else
            "---"
          end
        end
      )
    end
  end

  # サマリー表のAgent取得プロダクト
  def create_summary_license_agent_prds(pkg_count, app_count, host, osname, sidfm_roles)
    content = create_summary_license_apps(app_count, host[:sidfmd_host].id).concat(
      create_summary_license_pkgs(host, pkg_count, osname, sidfm_roles)
    )
  end

  # サマリー表の手動登録プロダクト変更ボタン
  def create_summary_license_change_manual_apps_button(host, lapp_count)
    content = content_tag(
      :div
    ) do
      concat(
        button_tag(
          "登録プロダクト変更",
          :type => "submit",
          :class => "btn btn-default btn-xs btn-modifyapp text-nowrap my-1",
          :role => "button",
          :autocomplete => "off",
          :data => {
            :toggle => "modal",
            :target => "#modalModifyApp",
            :hostname => "#{host[:hostname]}",
            :displayname => "#{host[:sidfmd_host] ?  host[:sidfmd_host].display_name : host[:hostname]}",
            :lappcount => lapp_count
          }
        )
      )
    end
  end

  # サマリー表の手動登録プロダクト
  def create_summary_license_manual_apps(host, lapp_count, sidfm_roles)
    content = content_tag(
      :div
    ) do
      concat(
        if lapp_count > 0 then
          content_tag(
            :div,
            :class => "balloon-dropdown"
          ) do
            concat(
              button_tag(
                :type => "button",
                :class => "btn btn-xs btn-light-nb2 text-nowrap action-open-list create-balloon dropdown-without-arrow text-nowrap dropdown-toggle caret-off",
                :data => { :lappsize => lapp_count, :showlist => "lapps", :hostid => host[:sidfmd_host].id },
              ) do
                concat(
                  content_tag(
                    :span,
                    :class => "copy-tooltip-btn",
                    :data => { :toggle => "tooltip", :placement => "bottom", :title => "コピーしました", :trigger => "manual" }
                  ) do
                    concat( content_tag(:span, "#{lapp_count}", :class => "badge badge-pill badge-gray-999") )
                  end
                )
              end
            )
            .concat(
              content_tag(
                :div,
                :class => "dropdown-menu invisible",
                :style => "width: 300px;",
                :data => { :listmenu => true }
              ) do
                concat(
                  content_tag(
                    :div,
                    :class => "balloon-menu",
                    :style => "padding-bottom:0px !important; max-width:330px !important;"
                  ) do
                    concat(
                      content_tag(:div, nil, :class => "balloon-arrow")
                    )
                    .concat(
                      content_tag(
                        :div,
                        :class => "dropdown-item bg-white",
                        :style => "overflow-x: hidden;"
                      ) do
                        concat(
                          content_tag(
                            :div,
                            nil,
                            :id => "manual-list#{host[:sidfmd_host].id}",
                            :class => "list-group"
                          )
                        )
                      end
                    )
                    .concat(
                      content_tag(
                        :div, :class => "dropdown-item bg-white"
                      ) do
                        concat(
                          content_tag(
                            :div,
                            :class => "float-right mb-1",
                            :style => "margin-top: 1px;"
                          ) do
                            concat(
                              button_tag(
                                "コピー",
                                :id => "copy-manual-list#{host[:sidfmd_host].id}",
                                :class => "btn btn-primary btn-xs g-font-size-0_74x copybtn text-nowrap",
                                "data-clipboard-target" => "#manual-list#{host[:sidfmd_host].id}"
                              )
                            )
                          end
                        )
                      end
                    )
                  end
                )
              end
            )
          end
        else
          content_tag(:span, "---", :class => "g-font-size-13")
        end
      )
    end
    if sidfm_roles.include?(:host_admin) or sidfm_roles.include?(:host_limited_admin) then
      content = content.concat( create_summary_license_change_manual_apps_button(host, lapp_count) )
    end
    content
  end

  # サマリー表の基準日
  def create_summary_license_basedate(host, sidfm_roles)
    content = content_tag(
      :div
    ) do
      concat(
        content_tag(
          :span,
          "#{(host[:sidfmd_host] && host[:sidfmd_host].refdate) ? host[:sidfmd_host].refdate.strftime('%Y/%m/%d') : "---"}",
          :class => "g-font-size-13"
        )
      )
    end
    if sidfm_roles.include?(:host_admin) then
      content = content.concat(
        button_tag(
          "基準日変更",
          :type => "submit",
          :class => "btn btn-default btn-xs text-nowrap my-1",
          :role => "button",
          :autocomplete => "off",
          :data => {
            :toggle => "modal",
            :target => "#refdateModal",
            :hostname => "#{host[:hostname]}",
            :refdate => "#{host[:sidfmd_host] ? (host[:sidfmd_host].refdate ? host[:sidfmd_host].refdate.strftime('%Y/%m/%d') : '') : ''}",
            :title => "基準日変更 - #{host[:sidfmd_host] ? host[:sidfmd_host].display_name : host[:hostname]}"
          }
        )
      )
    end
    content
  end

  # サマリー表のメモ
  def create_summary_license_memo(host, sidfm_roles)
    content = 
      if host[:sidfmd_host] then
        ret = ""
        if host[:sidfmd_host].memo then
          ret = content_tag(
            :div,
            :class => "balloon-dropdown d-inline-block",
            :style => "margin-bottom:-0.25rem;"
          ) do
            concat(
              button_tag(
                :type => "button",
                :class => "create-balloon dropdown-without-arrow btn btn-xs btn-light-nb2 text-nowrap my-1 dropdown-toggle caret-off",
              ) do
                concat(
                  content_tag(
                    :span,
                    :class => "d-inline-flex"
                  ) do
                    concat( content_tag(:span, "記載有", :style => "overflow:hidden;") )
                  end
                )
              end
            )
            .concat(
              content_tag(
                :div,
                :class => "dropdown-menu"
              ) do
                concat(
                  content_tag(
                    :div,
                    :class => "balloon-menu"
                  ) do
                    concat(
                      content_tag(:div, nil, :class => "balloon-arrow")
                    )
                    .concat(
                      content_tag(
                        :div,
                        "#{host[:sidfmd_host].memo}",
                        :class => "dropdown-item balloon-host-item bg-white text-dark"
                      )
                    )
                  end
                )
              end
            )
          end
        else
          ret = content_tag(:div, "---", :class => "g-font-size-13")
        end
        if sidfm_roles.include?(:host_admin) or sidfm_roles.include?(:host_limited_admin) then
          ret = ret.concat(
            button_tag(
              "メモ更新",
              :type => "submit",
              :class => "btn btn-default btn-xs text-nowrap my-1",
              :role => "button",
              :autocomplete => "off",
              :data => {
                :toggle => "modal",
                :target => "#memoModal",
                :hostname => "#{host[:hostname]}",
                :memo => "#{host[:sidfmd_host] ? (host[:sidfmd_host].memo ? host[:sidfmd_host].memo : '') : ''}",
                :title => "メモ更新 - #{host[:sidfmd_host] ? host[:sidfmd_host].display_name : host[:hostname]}"
              }
            )
          )
        end
        ret
      else
        content_tag(:div, "---", :class => "g-font-size-13")
      end
  end

  #サマリー表の操作
  def create_summary_license_operation(host, license_number, used_license_number)
    del_rev = host[:flg_delete] ? '復元' : '削除'
    content = 
      if license_number <= used_license_number and host[:flg_delete] then
        button_tag(
          "ホストの#{del_rev}",
          :type => 'button',
          :class => "btn #{host[:flg_delete] ? 'btn-secondary' : 'btn-danger'} btn-xs text-nowrap",
          :role => "button",
          :disabled => 'disabled',
          :title => 'ライセンスの登録ホスト数制限に達しています。'
        )
      else
        form_tag(
          { :controller => :sidfm_licenses, :action => :toggle_host_delete },
          :method => :post,
          :class => 'dump-pages-confirm',
          :data => (
            host[:flg_delete] ?
            { :target => "#dump_id_#{host[:id]}", :confirm => "#{del_rev}してよろしいですか？" } : 
            { :formdel => "host_#{host[:sidfmd_host] ? host[:sidfmd_host].id : "0"}"}
          )
        ) do
          concat( hidden_field_tag("params[id]", host[:id], :id => "params_id_#{host[:id]}") )
          .concat( hidden_field_tag("params[hostname]", "#{host[:hostname]}", :id => "hostname_id_#{host[:id]}") )
          .concat( hidden_field_tag("params[dump]", {}.to_json, :id => "dump_id_#{host[:id]}") )
          .concat(
            button_tag(
              "ホストの#{del_rev}",
              :type => "button",
              :name => "commit",
              :class => "btn #{host[:flg_delete] ? 'btn-secondary' : 'btn-danger'} btn-xs text-nowrap",
              :role => "button",
              :data => {
                :hostid => "#{host[:id]}",
                :sidfmdhostid => "#{host[:sidfmd_host] ? host[:sidfmd_host].id : "0"}",
                "#{host[:flg_delete] ? "undelhost" : "delhost"}" => "true"
              }
            )
          )
        end
      end
  end


end
