#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmRegkeysHelper
  
  # サマリー表のチェックボックス
  def create_summary_regkey_chkbox(apikey_id, checked)
    content = check_box_tag(
      nil,
      apikey_id,
      checked.include?(apikey_id),
      :class => 'SelectApikey',
      :id => "summary-apikey#{apikey_id}"
    )
  end

  # 登録キー名
  def create_summary_regkey_name(apikey, sidfm_roles)
    content = content_tag(
      :div,
      :class => "summary-apikey-name d-flex"
    ) do
      concat(
        content_tag(
          :span,
          "#{apikey.name}",
          :class => "mr-auto name",
          :data => { :toggle => "tooltip", :placement => "right", :title => "#{apikey.name}" }
        )
        .concat(    
          button_tag(
            'キー名変更',
            :type => 'button',
            :class => 'ml-auto update-button btn btn-default btn-xs text-nowrap',
            :data => { :updateapikey => true, :update => :name, :id => apikey.id, :content => apikey.name },
            :disabled => !sidfm_roles.include?(:admin)
          )
        )
      )
    end
  end

  # APIキー
  def create_regkey_apikey(apikey, is_enddate_passed, item_name)
    content = content_tag(
      :div,
      :class => "d-flex"
    ) do
      concat(
        content_tag(
          :span,
          "#{item_name}",
          :class => "apikey-label"
        )
      )
      .concat(
        content_tag(
          :div,
          :class => "apikey-value d-flex"
        ) do
          concat(
            if apikey.api_key then
              content_tag(
                :div,
                "#{apikey.api_key}",
                :class => (is_enddate_passed ? "text-danger" : "")
              )
              .concat(
                content_tag(
                  :i,
                  nil,
                  :class => "far fa-copy ml-auto pl-1 copy-apikey cursor-pointer",
                  :data => {
                    "clipboard-text" => "#{apikey.api_key}",
                    :toggle => "tooltip",
                    :placement => "bottom",
                    :title => "コピーしました!",
                    :trigger => "manual"
                  },
                  :style => "color:#605e5e;"
                )
              )
            else
              content_tag(
                :span,
                "未登録",
                :class => "slanted-font#{is_enddate_passed ? ' text-danger' : ''}"
              )
            end
          )
        end
      )
    end
  end

  # サマリー表のAPIキー
  def create_summary_regkey_apikey(apikey, is_enddate_passed, item_name, sidfm_roles)
    has_c_cats = apikey.sidfmd_regkey_cats.joins(:sidfmd_category)
                                          .where(:sidfmd_categories => { :is_system => false })
                                          .exists?
    content = content_tag(
      :div,
      :class => "summary-apikey apikey-block apikey-item"
    ) do
      concat(
        content_tag(
          :div,
          :class => "apikey"
        ) do
          concat(
            create_regkey_apikey(apikey, is_enddate_passed, item_name)
          )
          .concat(
            content_tag(
              :div,
              :class => "ml-auto d-flex"
            ) do
              concat(
                form_tag(
                  { :controller => :sidfm_regkeys, :action => :modify_host_apikey },
                  :method => :post,
                  :data => { :formid => 'form-update-apikey-key' }
                ) do
                  concat(
                    if has_c_cats then
                      button_tag(
                        (apikey.api_key ? 'APIキー更新' : 'APIキー登録'),
                        :type => 'button',
                        :class => "update-button btn btn-default btn-xs text-nowrap exec-update-apikey-key",
                        :data => { :confirm => "「#{apikey.name}」のAPIキーを#{apikey.api_key ? '更新' : '登録'}します" },
                        :disabled => !sidfm_roles.include?(:admin)
                      )
                      .concat(hidden_field_tag('form[target]', 'api_key', :id => "update-apikey-target-#{apikey.id}"))
                      .concat(hidden_field_tag('form[update_api_key]', true, :id => ''))
                      .concat(hidden_field_tag('form[id]', apikey.id, :id => "update-apikey-key-id-#{apikey.id}"))
                      .concat(hidden_field_tag('form[page_info]', '{}', :id => "update-apikey-page-info-#{apikey.id}"))
                    else
                      button_tag(
                        (apikey.api_key ? 'APIキー更新' : 'APIキー登録'),
                        :type => 'button',
                        :class => "update-button disabled btn btn-default btn-xs text-nowrap exec-update-apikey-key",
                        :data => {
                          :toggle => 'tooltip',
                          :title => "APIキーの#{apikey.api_key ? '更新' : '登録'}には、カテゴリの登録が必要です"
                        },
                        :disabled => 'disabled'
                      )
                    end
                  )
                end
              )
              .concat(
                if apikey.api_key then
                  form_tag(
                    { :controller => :sidfm_regkeys, :action => :modify_host_apikey },
                    :method => :post,
                    :data => { :formid => 'form-update-apikey-key' }
                  ) do
                    concat(
                      button_tag(
                        'APIキー削除',
                        :type => 'button',
                        :class => 'ml-1 update-button btn btn-danger btn-xs text-nowrap exec-update-apikey-key',
                        :data => { :confirm => "「#{apikey.name}」のAPIキーを削除します" },
                        :disabled => !sidfm_roles.include?(:admin)
                      )
                      .concat(hidden_field_tag('form[target]', 'api_key', :id => "remove-apikey-target-#{apikey.id}"))
                      .concat(hidden_field_tag('form[remove_api_key]', true, :id => ''))
                      .concat(hidden_field_tag('form[id]', apikey.id, :id => "remove-apikey-key-id-#{apikey.id}"))
                      .concat(hidden_field_tag('form[page_info]', '{}', :id => "remove-apikey-page-info-#{apikey.id}"))
                    )
                  end
                else
                  ""
                end
              )
            end
          )
        end
      )
    end
  end
 
  # サマリー表のホスト名接頭辞
  def create_summary_regkey_hostname_prefix(apikey, item_name, available_agent_versions, sidfm_roles)
    content = content_tag(
      :div,
      :class => "hostname-prefix summary-apikey-block-item left apikey-item"
    ) do
      concat(
        content_tag(
          :span,
          :class => "apikey-label"
        ) do
          concat(
            content_tag(:span, "#{item_name}")
          )
          .concat(
            content_tag(
              :a,
              :data => {
                :toggle => "tooltip",
                :placement => "right",
                :html => "true",
                :title => 'ホスト登録時に、ホスト名に付与される接頭辞<div class="mt-1">※この機能に対応しているSIDfmVMエージェントのバージョンは、以下のバージョンのみです。<br>' + available_agent_versions.join('<br>') + '</div>'
              }
            ) do
              concat( content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70 ml-1px") )
            end
          )
        end
      )
      .concat(
        content_tag(
          :span,
          :class => "apikey-value limit-width-value",
          :data => { :toggle => "tooltip", :placement => "right", :title => "#{apikey.hostname_prefix}" }
        ) do
          concat(       
            if apikey.hostname_prefix.size > 0 then
              content_tag(:span, "#{apikey.hostname_prefix}")
            else
              content_tag(:span, 'なし', :class => "slanted-font")
            end
          )
        end
      )
      .concat(
        button_tag(
          '接頭辞変更',
          :type => 'button',
          :class => 'ml-auto update-button btn btn-default btn-xs text-nowrap',
          :data => { :updateapikey => true, :update => :hostname_prefix, :id => apikey.id, :content => apikey.hostname_prefix },
          :disabled => !sidfm_roles.include?(:admin)
        )
      )
    end
  end

  # サマリー表のホスト名接尾辞
  def create_summary_regkey_hostname_suffix(apikey, item_name, available_agent_versions, sidfm_roles)
    content = content_tag(
      :div,
      :class => "hostnum-suffix summary-apikey-block-item apikey-item"
    ) do
      concat(
        content_tag(
          :span,
          :class => "apikey-label"
        ) do
          concat(
            content_tag(:span, "#{item_name}")
          )
          .concat(
            content_tag(
              :a,
              :data => {
                :toggle => "tooltip",
                :placement => "right",
                :html => "true",
                :title => 'ホスト登録時に、ホスト名に付与される接尾辞<div class="mt-1">※この機能に対応しているSIDfmVMエージェントのバージョンは、以下のバージョンのみです。<br>' + available_agent_versions.join('<br>') + '</div>'
              }
            ) do
              concat( content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70 ml-1px") )
            end
          )
        end
      )
      .concat(
        content_tag(
          :span,
          :class => "apikey-value limit-width-value",
          :data => { :toggle => "tooltip", :placement => "right", :title => "#{apikey.hostname_suffix}" }
        ) do
          concat(
            if apikey.hostname_suffix.size > 0 then
              content_tag(:span, "#{apikey.hostname_suffix}")
            else
              content_tag(:span, "なし", :class => "slanted-font")
            end
          )
        end
      )
      .concat(
        button_tag(
          '接尾辞変更',
          :type => 'button',
          :class => 'ml-auto update-button btn btn-default btn-xs text-nowrap',
          :data => { :updateapikey => true, :update => :hostname_suffix, :id => apikey.id, :content => apikey.hostname_suffix },
          :disabled => !sidfm_roles.include?(:admin)
        )
      )
    end
  end

  # サマリー表のホスト名接頭辞・接尾辞 の行
  def create_summary_regkey_hostname_pre_suffix(apikey, hostname_prefix_item_name, hostname_suffix_item_name, available_agent_versions, sidfm_roles)
    content = content_tag(
      :div,
      :class => "summary-apikey-settings-col"
    ) do
      concat( create_summary_regkey_hostname_prefix(apikey, hostname_prefix_item_name, available_agent_versions, sidfm_roles) )
      .concat( create_summary_regkey_hostname_suffix(apikey, hostname_suffix_item_name, available_agent_versions, sidfm_roles) )
    end
  end

  # サマリー表の「同ホスト名が登録済の場合」
  def create_summary_regkey_dup_action(apikey, value_text, item_name, available_agent_versions, sidfm_roles)
    content = content_tag(
      :div,
      :class => "dup-action summary-apikey-block-item left apikey-item"
    ) do
      concat(
        content_tag(
          :span,
          :class => "apikey-label"
        ) do
          concat(
            content_tag(:span, "#{item_name}")
          )
          .concat(
            content_tag(
              :a,
              :data => {
                :toggle => "tooltip",
                :placement => "right",
                :html => "true",
                :title => "ホスト登録時に、既に同じホスト名が登録されていた場合の挙動#{apikey.dup_action.to_sym == :num_suffix ? '<div>※' + "「#{SidfmdRegkey::DUP_ACTION_NUM_SUFFIX_LABEL}」" + 'を選択してホストを登録するには、 以下のSIDfmVMエージェントを利用する必要があります。<br>' + available_agent_versions.map{|v| "・#{v}"}.join('<br>') + '</div>' : ''}"
              }
            ) do
              concat( content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70 ml-1px") )
            end
          )
        end
      )
      .concat(
        content_tag(:span, "#{value_text}", :class => "apikey-value")
      )
      .concat(
        button_tag(
          '設定変更',
          :type => 'button',
          :class => 'ml-auto update-button btn btn-default btn-xs text-nowrap',
          :data => { :updateapikey => true, :update => :dup_action, :id => apikey.id, :content => apikey.dup_action },
          :disabled => !sidfm_roles.include?(:admin)
        )
      )
    end
  end

  # サマリー表の「表示名を設定」
  def create_summary_regkey_is_dn_orig(apikey, item_name, sidfm_roles)
    content = content_tag(
      :div,
      :class => "is-dn-orig summary-apikey-block-item apikey-item"
    ) do
      concat(
        content_tag(
          :span,
          :class => "apikey-label"
        ) do
          concat(
            content_tag(:span, "#{item_name}")
          )
          .concat(
            content_tag(
              :a,
              :data => {
                :toggle => "tooltip",
                :placement => "right",
                :html => "true",
                :title => 'ホスト登録時に、表示名を設定するか否か<div class="mt-t">※ 設定される表示名は、接頭辞・接尾辞を含まない元のホスト名になります。</div>'
              }
            ) do
              concat( content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70 ml-1px") )
            end
          )
        end
      )
      .concat(
        content_tag(:span, "#{apikey.is_dn_orig ? '設定する' : '設定しない'}", :class => "apikey-value")
      )
      .concat(
        button_tag(
          '設定変更',
          :type => 'button',
          :class => 'ml-auto update-button btn btn-default btn-xs text-nowrap',
          :data => { :updateapikey => true, :update => :is_dn_orig, :id => apikey.id, :content => apikey.is_dn_orig },
          :disabled => !sidfm_roles.include?(:admin)
        )
      )
    end
  end

  # サマリー表の「同ホスト名が登録済の場合」・「表示名を設定」の行
  def create_summary_regkey_hostname_options(apikey, dup_action_value_text, dup_action_item_name, is_dn_orig_item_name, available_agent_versions, sidfm_roles)
    content = content_tag(
      :div,
      :class => "summary-apikey-settings-col"
    ) do
      concat( 
        create_summary_regkey_dup_action(
          apikey,
          dup_action_value_text, 
          dup_action_item_name,
          available_agent_versions,
          sidfm_roles
        )
      )
      .concat(
        create_summary_regkey_is_dn_orig(apikey, is_dn_orig_item_name, sidfm_roles)
      )
    end
  end

  # サマリー表の制限ホスト数
  def create_summary_regkey_hostnum_limit(apikey, item_name, sidfm_roles)
    content = content_tag(
      :div,
      :class => "hostnum-limit d-flex align-items-center apikey-item"
    ) do
      concat(
        content_tag(
          :span,
          :class => "apikey-label"
        ) do
          concat(
            content_tag(:span, "#{item_name}")
          )
          .concat(
            content_tag(
              :a,
              :data => { :toggle => "tooltip", :placement => "right", :html => "true", :title => "カテゴリによるホスト数の制限" }
            ) do
              concat( content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70 ml-1px") )
            end
          )
        end
      )
      .concat(
        content_tag(
          :span,
          "#{apikey.hostnum_limit >= 0 ? apikey.hostnum_limit : '無制限'}",
          :class => "apikey-value limit-width-value",
          :data => {
            :toggle => "tooltip",
            :placement => "right",
            :title => "#{apikey.hostnum_limit >= 1000000 ? apikey.hostnum_limit : ""}"
          }
        )
      )
      .concat(
        button_tag(
          '制限数変更',
          :type => 'button',
          :class => 'ml-auto update-button btn btn-default btn-xs text-nowrap',
          :data => { :updateapikey => true, :update => :hostnum_limit, :id => apikey.id, :content => apikey.hostnum_limit },
          :disabled => !sidfm_roles.include?(:admin)
        )
      )
    end
  end

  # カテゴリ
  def create_regkey_categories(apikey, user_category_ids)
    apikey_cats = apikey.sidfmd_regkey_cats
    if !user_category_ids.nil? then
      apikey_cats = apikey_cats.where("sidfmd_category_id IN (?) OR sidfmd_category_id IN (?)",
        user_category_ids, SidfmdCategory.where(:is_system => true).pluck(:id))
    end
    apikey_cats_count = apikey_cats.count
    cats = apikey_cats.joins(:sidfmd_category)
    content =
      if apikey_cats_count == 0 then
        content_tag(:span, "未登録", :class => "slanted-font")
      elsif apikey_cats_count == 1 then
        system_cat = apikey_cats.first.sidfmd_category
        content_tag(
          :span,
          :class => "align-middle text-left g-font-size-0_86x limit-width-value",
          :data => { :menu => "apikey-categories" }
        ) do
          concat(
            link_to(
              system_cat.name,
              '#',
              :data => {
                :toggle => 'tooltip',
                :title => system_cat.desc,
                :boundary => 'window',
                :placement => 'right',
                :filteritem => 'category',
                :name => 'category',
                :value => system_cat.id
              }
            )
          )
        end
      else
        system_cat = cats.where(:sidfmd_categories => { :is_system => true })
                         .select(:name, :desc, 'sidfmd_categories.id AS id')
                         .first
        content_tag(
          :div,
          :class => "d-inline-block balloon-dropdown"
        ) do
          concat(
            button_tag(
              :type => "button",
              :class => "create-balloon text-left dropdown-without-arrow btn btn-xs btn-light-nb2 align-middle my-1 text-nowrap",
              :style => "font-size: 83% !important;"
            ) do
              concat(
                content_tag(
                  :span,
                  :class => "d-inline-flex",
                  :style => "width:6em!important;"
                ) do
                  concat(
                    content_tag(:span, system_cat.name, :style => "overflow:hidden;")
                  )
                end
              )
              .concat(
                content_tag(:span, "#{cats.size}", :class => "badge badge-pill badge-gray-999", :style => "margin-left: 1px;")
              )
            end
          )
          .concat(
            content_tag(
              :div,
              :class => "dropdown-menu",
              :data => { :menu => "apikey-categories" }
            ) do
              concat(  
                content_tag(
                  :div,
                  :class => "balloon-menu"
                ) do
                  concat(
                    content_tag(:div, nil, :class => "balloon-arrow")
                  )
                  .concat(
                    content_tag(
                      :div,
                      :class => "dropdown-item"
                    ) do
                      concat(
                        content_tag(
                          :div,
                          :class => "list-group"
                        ) do
                          cats.order('CASE WHEN is_system IS TRUE THEN sidfmd_category_id ELSE NULL END')
                              .order('LOWER(name COLLATE "C")')
                              .select(:name, :desc, 'sidfmd_categories.id AS id')
                              .each do |cat|
                                concat(
                                  link_to(
                                    cat.name,
                                    '#',
                                    :class => 'list-group-item text-truncate g-max-width-16em',
                                    :data => {
                                      :filteritem => 'category',
                                      :name => 'category',
                                      :value => cat.id,
                                      :toggle => 'tooltip',
                                      :title => cat.desc,
                                      :placement => 'right',
                                      :boundary => 'window'
                                    }
                                  )
                                )
                              end
                        end
                      )
                    end
                  )
                end
              )
            end
          )
        end
      end
  end

  # サマリー表のカテゴリ
  def create_summary_regkey_categories(apikey, item_name, sidfm_roles, user_category_ids)
    apikey_c_cat_ids = apikey.sidfmd_regkey_cats
      .joins(:sidfmd_category)
      .where(:sidfmd_categories => { :is_system => false })
      .pluck(:sidfmd_category_id)
    if !user_category_ids.nil? then
      apikey_c_cat_ids = apikey_c_cat_ids & user_category_ids
    end
    content = content_tag(
      :div,
      :class => "categories d-flex align-items-center apikey-item"
    ) do
      concat(
        content_tag(
          :span,
          :class => "apikey-label"
        ) do
          concat(
            content_tag(:span, "#{item_name}")
          )
          .concat(
            content_tag(
              :a,
              :data => { :toggle => "tooltip", :placement => "right", :html => "true", :title => "ホスト登録時に、ホストに設定されるカテゴリ" }
            ) do
              concat( content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70") )
            end
          )
        end
      )
      .concat(
        content_tag(
          :div,
          :class => "apikey-value"
        ) do
          concat( create_regkey_categories(apikey, user_category_ids) )
        end
      )
      .concat(
        content_tag(
          :div,
          :class => "ml-auto",
        ) do
          concat(
            button_tag(
              'カテゴリ変更',
              :type => 'button',
              :class => 'update-button btn btn-default btn-xs text-nowrap',
              :data => {
                :updateapikey => true,
                :update => :categories,
                :id => apikey.id,
                :content => {
                  :system_category_id => apikey.sidfmd_regkey_cats
                                               .joins(:sidfmd_category)
                                               .where(:sidfmd_categories => { :is_system => true })
                                               .pluck(:sidfmd_category_id)
                                               .first,
                  :custom_category_ids => apikey_c_cat_ids
                }.to_json
              },
              :disabled => !sidfm_roles.include?(:admin)
            )
          )
        end
      )
    end
  end

  # サマリー表の有効期限
  def create_summary_regkey_period(apikey, is_enddate_passed, item_name, sidfm_roles)
    begin_date = apikey.begin_date ? SidfmdRegkey.date_to_string(apikey.begin_date) : nil
    end_date = apikey.end_date ? SidfmdRegkey.date_to_string(apikey.end_date) : nil

    content = content_tag(
      :div,
      :class => "period summary-apikey-block-item apikey-item"
    ) do
      concat(
        content_tag(
          :span,
          :class => "apikey-label"
        ) do
          concat(
            content_tag(:span, "#{item_name}")
          )
          .concat(
            content_tag(
              :a,
              :data => { :toggle => "tooltip", :placement => "right", :html => "true", :title => "APIキーの有効期限" }
            ) do
              concat( content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70 ml-1px") )
            end
          )
        end
      )
      .concat(
        content_tag(
          :span,
          :class => "apikey-value"
        ) do
          concat(
            if begin_date and end_date then
              content_tag(
                :span,
                "#{begin_date == end_date ? "#{begin_date} のみ有効" : "#{begin_date} 〜 #{end_date}"}",
                :class => (is_enddate_passed ? "text-danger" : "")
              )
            elsif begin_date then
              content_tag(:span, "#{begin_date} 以降有効")
            elsif end_date then
              content_tag(:span, "#{end_date} まで有効", :class => (is_enddate_passed ? "text-danger" : ""))
            else
              content_tag(:span, "無期限")
            end
          )
        end
      )
      .concat( 
        content_tag(
          :span,
          :class => "ml-auto"
        ) do
          concat(
            button_tag(
              '有効期間変更',
              :type => 'button',
              :class => 'update-button btn btn-default btn-xs text-nowrap',
              :data => {
                :updateapikey => true,
                :update => :period,
                :id => apikey.id,
                :content => { :begin_date => begin_date, :end_date => end_date }.to_json
              },
              :disabled => !sidfm_roles.include?(:admin)
            )
          )
        end
      )
    end
  end

  # サマリー表の制限ホスト数とカテゴリと有効期限
  def create_summary_regkey_options(apikey, is_enddate_passed, hostnum_limit_item_name, categories_item_name, period_item_name, sidfm_roles, user_category_ids)
    content = content_tag(
      :div,
      :class => "summary-apikey-settings-col over-three-items"
    ) do
      concat(
        content_tag(
          :div,
          :class => "summary-apikey-block-item left two-items"
        ) do
          concat(
            create_summary_regkey_hostnum_limit(apikey, hostnum_limit_item_name, sidfm_roles)
          )
          .concat(
            create_summary_regkey_categories(apikey, categories_item_name, sidfm_roles, user_category_ids)
          )
        end
      )
      .concat( create_summary_regkey_period(apikey, is_enddate_passed, period_item_name, sidfm_roles) )
    end
  end

  # サマリー表の登録キー 
  def create_summary_regkey_content(apikey, item_names, dup_action_names, available_agent_versions, sidfm_roles, user_category_ids)
    is_enddate_passed = (apikey.end_date and apikey.end_date < Date.today) #true/falseがタイミングによって変わる可能性が0ではないので、ここで固定する
    content = content_tag(
      :div,
      :class => "summary-apikey-card"
    ) do
      concat( create_summary_regkey_name(apikey, sidfm_roles) )
      .concat(
        content_tag(
          :div,
          :class => "summary-apikey-content"
        ) do
          concat(
            create_summary_regkey_apikey(apikey, is_enddate_passed, item_names[:api_key], sidfm_roles)
          )
          .concat(
            create_summary_regkey_hostname_pre_suffix(
              apikey,
              item_names[:hostname_prefix],
              item_names[:hostname_suffix],
              available_agent_versions,
              sidfm_roles
            )
          )
          .concat(
            create_summary_regkey_hostname_options(
              apikey,
              dup_action_names[apikey.dup_action.to_sym],
              item_names[:dup_action],
              item_names[:is_dn_orig],
              available_agent_versions,
              sidfm_roles
            )
          )
          .concat(
            create_summary_regkey_options(
              apikey,
              is_enddate_passed,
              item_names[:hostnum_limit],
              item_names[:categories],
              item_names[:period],
              sidfm_roles,
              user_category_ids
            ) 
          )
        end
      )
    end
  end

  # サマリー表の操作
  def create_summary_regkey_operation(apikey, sidfm_roles)
    content = form_tag(
      { :controller => :sidfm_regkeys, :action => :delete_host_register_apikey },
      :method => :post,
      :data => { :form => 'delete' }
    ) do
      concat(
        button_tag(
          '削除',
          :type => 'submit',
          :class => 'btn btn-xs btn-danger delete text-nowrap',
          :data => { :confirm => "「#{apikey.name}」を削除してもよろしいですか?" },
          :disabled => !sidfm_roles.include?(:admin)
        )
      )
      .concat( hidden_field_tag('form[id]', apikey.id, :id => "delete-form-#{apikey.id}") )
    end
  end

end
