#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdIssueCtag < ActiveRecord::Base
  belongs_to :sidfmd_issue
  belongs_to :sidfmd_ctag

  # 指定したSidfmdIssue(s)のSidfmdIssueCtagを、指定したctag_ids(SidfmdCtagのids)の紐づきに更新
  # ※Viewの更新があるため、呼びだし元ではtransactionで囲わずに、エラーハンドリングだけする。
  def self.udpate_issue_ctags(ctag_ids:, sidfmd_issue_rows:, role_admin:, user_category_ids:, user_host_ids:)
    #logger.info('ctag_ids: ' + ctag_ids.inspect)
    #logger.info('sidfmd_issue_rows: ' + sidfmd_issue_rows.inspect)
    #logger.info('role_admin: ' + role_admin.inspect)
    #logger.info('user_category_ids: ' + user_category_ids.inspect)
    #logger.info('user_host_ids: ' + user_host_ids.inspect)

    user_ctag_ids = nil
    ctag_ids = ctag_ids.map(&:to_i).uniq
    unless role_admin then
      sidfmd_issue_rows = sidfmd_issue_rows.where(:sidfmd_host_id => user_host_ids)
      user_ctag_ids = SidfmdCtag.user_relatable_ctag_ids(role_admin, user_category_ids)
      if (ctag_ids - user_ctag_ids).any? then
        logger.error("user can not edit ids: #{(ctag_ids - user_ctag_ids).inspect}")
        raise Exception
      end
    end
    issues_ids = []
    update_data = []
    sidfmd_issue_rows.each do |issue|
      update_data << {
        :sidfmd_issue_id => issue.id,
        :html_id => issue.html_id,
        :sidfmd_host_id => issue.sidfmd_host_id,
      }
      issues_ids << issue.id
    end
    new_issue_ctags = []
    ctag_ids.each do |ctag_id|
      issues_ids.each do |id|
        new_issue_ctags << SidfmdIssueCtag.new(sidfmd_issue_id: id, sidfmd_ctag_id: ctag_id)
      end
    end

    del_issue_ctags = SidfmdIssueCtag.where(:sidfmd_issue_id => issues_ids)
    unless role_admin then
      del_issue_ctags = del_issue_ctags.where(:sidfmd_ctag_id => user_ctag_ids)
    end
    SidfmdIssueCtag.transaction do
      del_issue_ctags.destroy_all
      SidfmdIssueCtag.import(new_issue_ctags)
    end
    #View更新
    #SidfmdVulView.sync_data(SidfmdHostView.relate_htmlids(update_data.map{|i| i[:sidfmd_host_id]}))
    #SidfmdHostView.sync_data(SidfmdVulView.relate_hostids(update_data.map{|i| i[:html_id]}))
    SidfmdVulView.update_from_rule_action(update_data)
    SidfmdHostView.update_from_rule_action(update_data)
  end
end
