/* coding: utf-8 */

//BootStrapTableのtdのスタイル。画面ごとに関数を作って設定する
function BootStrapTableCellStyle(value, row, index, field) {
    switch (field){
        case 'chkbox':
            return { classes: 'summary-chk-box-td' };
        case 'content':
            return { classes: 'summary-apikey-td' };
        case 'operation':
            return { classes: 'summary-apikey-action-td overflow-auto' };
        default:
            return { classes: '' };
    }
}

$(function() {
    activate_sidebar_item('sidebar-item-vmapikeyadmin-manip');

    const summaryTableId = 'vm-apikeys-summary-table';
    const summaryTableElm = $('#' + summaryTableId);
    const pageName = 'vm_apikey';
    const searchUsersPath = $('#sidfm_internal_api_search_users_path').val();

    const createSearchedUserElms = (str, users) => {
        return users.map(user => {
            const hitStr = user.login.toLowerCase().indexOf(str.toLowerCase()) !== -1 ?
                user.login :
                (user.lastname + ' ' + user.firstname)
            ;
            return $('<div>').attr({ 
                       'data-login': user.login,
                       'data-userid': user.id,
                       'data-toggle': 'tooltip',
                       'data-title': ('ログイン名: ' + escape_html_special_chars(user.login) + '<br>' +
                           '氏名: ' + escape_html_special_chars(user.lastname) + ' ' + escape_html_special_chars(user.firstname)
                       ),
                       'data-placement': 'right',
                       'data-boundary': 'window',
                       'data-html': true,
                       'class': 'searched-user-name',
                   }).text(hitStr);
        });
    };

    //削除
    const enableDeleteApikey = () => {
        $('#' + summaryTableId + ' [data-delete="apikey"]').on('click', function(){
            let conf = confirm($(this).attr('data-confirmtext'));
            if( conf ){
                dynamicSummaryTable.executeProcess({
                    paramsData: { id: $(this).attr('data-id') },
                    path: $('#apikey-delete-path').val(),
                    additionalAlertMessage: '削除に失敗しました。'
                });
            }
            return false;
        });
    };

    //更新
    const setModifyName = (triggerElm) => {
        const data = JSON.parse(triggerElm.attr('data-content'));
        $('#modify-apikey-name-field').val(data.name);
        $('#modify-apikey-item-name').show();
    };

    const setModifyUserItem = (triggerElm) => {
        const data = JSON.parse(triggerElm.attr('data-content'));
        const userData = data.user;
        $('#modify-apikey-search-user-field').val(userData.login).attr('data-userid', userData.user_id);
        $('#modify-apikey-item-user').show();
    };

    const setModifyAllowUpdateItem = (triggerElm) => {
        const data = JSON.parse(triggerElm.attr('data-content'));
        $('#modify-apikey-select-allow-update').val(String(data.value));
        $('#modify-apikey-item-allow-update').show();
    };

    const toggleModifyApikeyPeriodUnlimitElm = () => {
        $('#modify-apikey-period-begindate-field, #modify-apikey-period-enddate-field').prop('disabled', $('#modify-apikey-period-unlimit').prop('checked'));
    };

    const setModifyPeriodItem = (triggerElm) => {
        const data = JSON.parse(triggerElm.attr('data-content'));
        $('#modify-apikey-period-unlimit').prop('checked', !data.begin_date && !data.end_date);
        $('#modify-apikey-period-begindate-field').val(data.begin_date);
        $('#modify-apikey-period-enddate-field').val(data.end_date);
        toggleModifyApikeyPeriodUnlimitElm();
        $('#modify-apikey-item-period').show();
    };

    const updateApikey = (triggerElm) => {
        let conf = confirm(triggerElm.attr('data-confirmtext'));
        if( !conf ) return;
        if( conf ){
            const paramsData = {
                target: 'api_key',
                id: triggerElm.attr('data-id'),
            };
            if( triggerElm.attr('data-removekey') ){
                paramsData.remove_api_key = true;
            }else{
                paramsData.update_api_key = true;
            }
            dynamicSummaryTable.executeProcess({
                paramsData,
                path: $('#apikey-modify-path').val(),
                additionalAlertMessage: (paramsData.update_api_key ? '変更に失敗しました。' : '削除に失敗しました。')
            });
        }
        return false; 
    };

    const enableUpdateApikeyItems = () => {
        $('#' + summaryTableId + ' [data-updateapikey="true"]').on('click', function(){
            const updateItem = $(this).attr('data-update');
            if( updateItem == 'key' ){
                updateApikey($(this));
                return;
            }
            const modal = $('#modalModifyApikey');
            modal.find('.modify-apikey-item').hide();
            $('#modify-apikey-id').val($(this).attr('data-id'));
            const itemNames = JSON.parse($('#apikey-item-names').val());
            modal.find('.modal-title').text(itemNames[updateItem] + '変更');
            switch (updateItem){
                case 'name':
                    setModifyName($(this));
                    break;
                case 'user':
                    setModifyUserItem($(this));
                    break;
                case 'allow_update':
                    setModifyAllowUpdateItem($(this));
                    break;
                case 'period':
                    setModifyPeriodItem($(this));
                    break;
            };
            modal.modal('show');
        });
    };

    $('#modify-apikey-period-unlimit').on('change', toggleModifyApikeyPeriodUnlimitElm);

    const modifyApikeySearchUserInput = $('#modify-apikey-search-user-field');
    const modifySearchedUsersArea = $('#modify-apikey-searched-users');
    const resetModifySearchUsers = () => {
        modifySearchedUsersArea.find('[data-toggle="tooltip"]').tooltip('hide');
        modifySearchedUsersArea.html('');
        modifyApikeySearchUserInput.removeAttr('data-userid');
    };

    const bindModifySearchedUserEvent = (elm) => {
        elm.tooltip();
        elm.on('click', function(){
            resetModifySearchUsers();
            modifyApikeySearchUserInput.val($(this).attr('data-login')).attr('data-userid', $(this).attr('data-userid'));
        });
    };

    modifyApikeySearchUserInput.on('input', function(){
        const str = $(this).val();
        if( str ){
            $.post(searchUsersPath, { str }, function(users){
                resetModifySearchUsers();
                if(users.length && modifyApikeySearchUserInput.val()){
                    modifySearchedUsersArea.append(createSearchedUserElms(str, users));
                    bindModifySearchedUserEvent(modifySearchedUsersArea.find('[data-userid]'));
                }
            });
        }else{
            resetModifySearchUsers();
        }
    });

    const validateModifyApikeyValue = (target) => {
        let errMessage = '';
        if( target === 'name' ){
            if( !$('#modify-apikey-name-field').val() ){
                errMessage = '1文字以上入力して下さい';
            }
        }else if( target === 'user' ){
            if( !modifyApikeySearchUserInput.attr('data-userid') ){
                errMessage = 'ユーザが選択されておりません。\nユーザを検索後、表示されたユーザから選択して下さい。';
            }
        }else if( target === 'allow_update'){
            //更新許可が可能かどうかはapiでチェックする
        }else if( target === 'period' ){
            if( !$('#modify-apikey-period-unlimit').prop('checked') ){
                let beginDate = get_date($('#modify-apikey-period-begindate-field').val());
                let endDate = get_date($('#modify-apikey-period-enddate-field').val());
                if( beginDate === false || endDate === false ){
                    errMessage = '日付は、西暦4桁、及び月と日のある日付を入力して下さい。\n※数字は半角で入力して下さい。';
                }else if( !$('#modify-apikey-period-begindate-field').val() && !$('#modify-apikey-period-enddate-field').val() ){
                    errMessage = '有効期限が無期限でない場合は、日付を入力して下さい';
                }else if( beginDate && endDate ){
                    beginDate = new Date(beginDate);
                    endDate = new Date(endDate);
                    if( beginDate > endDate ){
                        errMessage = '開始日は、終了日以前の日付にして下さい';
                    }   
                }
            }
        }
        return errMessage;
    };

    $('#exec-modify-apikey').on('click', function(){
        const button = $(this);
        button.addClass('pointer-events-none');
        const paramsData = { id: $('#modify-apikey-id').val() };
        $('#modalModifyApikey [name]:visible').each((idx, elm) => {
            elm = $(elm);
            switch (elm.attr('name')){
                case 'name':
                    paramsData.name = elm.val();
                    paramsData.target = 'name';
                    break;
                case 'user':
                    paramsData.user_id = elm.attr('data-userid');
                    paramsData.target = 'user';
                    break;
                case 'allow_update':
                    paramsData.allow_update = elm.val();
                    paramsData.target = 'allow_update';
                    break;
                case 'begindate':
                    if( !elm.prop('disabled') ){
                        paramsData.begin_date = elm.val();
                        paramsData.target = 'period'; //有効期限変更
                    }
                    break;
                case 'enddate':
                    if( !elm.prop('disabled') ){
                        paramsData.end_date = elm.val();
                        paramsData.target = 'period'; //有効期限変更
                    }
                    break;
                case 'is_unlimit_period':
                    paramsData.is_unlimit_period = elm.prop('checked');
                    paramsData.target = 'period'; //有効期限変更
                    break;
            };
        });
        const errMsg = validateModifyApikeyValue(paramsData.target);
        if( !errMsg ){
            const exec = () => {
                dynamicSummaryTable.executeProcess({
                    paramsData,
                    path: $('#apikey-modify-path').val(),
                    additionalAlertMessage: '変更に失敗しました。'
                });
                button.removeClass('pointer-events-none');
            };
            (async () => {
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                    additionalAlertMessage: '削除に失敗しました。'
                });
                if( roleOrCategoryError ) return;
                if( (paramsData.target === 'allow_update' && paramsData.allow_update === 'true') || paramsData.target === 'user' ){
                    const path = $('#api-is-apikey-allowed-allow-update-true-path').val();
                    const data = { id: paramsData.id };
                    if( paramsData.target === 'user' ){
                        data.after_change_user_id = paramsData.user_id;
                    }
                    $.post(path, data, function(isAllowed){
                        if( !isAllowed ){
                            if( paramsData.target === 'user' ){
                                const itemNames = JSON.parse($('#apikey-item-names').val());
                                alert(`${itemNames.allow_update}の項目が「許可」のAPIキーには、SIDfmVM閲覧者権限のユーザを設定できません。`);
                            }else{
                                alert('SIDfmVM閲覧者権限のユーザが設定されているキーは、「許可」への変更はできません。');
                            }
                            button.removeClass('pointer-events-none');
                            return
                        }
                        exec();    
                    });
                }else{
                    exec();
                }
            })();
        }else{
            alert(errMsg);
            button.removeClass('pointer-events-none');
        }
    });

    //サマリー表の動的処理のため(bootstrap-tableのため)にbindする。でないとイベントが効かない。
    const bindSummaryTableRowEvents = () => {
        enable_bootstrap_actions('#' + summaryTableId);
        enableUpdateApikeyItems();
        enableDeleteApikey();
    };
    const dynamicSummaryTable = new DynamicSummaryTable({
        summaryTableId,
        bindSummaryTableRowEventsFunction: bindSummaryTableRowEvents,
        pageName,
        allFilteredIdsId: 'all-apikey-ids',
        chkboxSelector: `#${summaryTableId} .apikeySelect`,
        allChkboxSelector: `#${summaryTableId} .allApikeySelect`,
        bulkBtnSelector: '.action-btn',
    });

    set_style_for_ActionArea_on_filter_dropdown_event();
    sticky_control(summaryTableId, pageName);
    $('#pages_form').on('submit', function(){ set_summary_table_lines(pageName, $('#pages_per_page').val()) });

    //フィルタ
    const summaryTableFilter = new SummaryTableFilter({dynamicSummaryTable});

    summaryTableFilter.enableFilterText(
        '#filter-name',
        '#reset-filter-name',
        { fieldSelector: '#filter-name-field' },
        'name'
    ); //APIキー名

    summaryTableFilter.enableFilterText(
        '#filter-username',
        '#reset-filter-username',
        { fieldSelector: '#filter-username-field' },
        'username'
    ); //ユーザ名

    summaryTableFilter.enableFilterChangeLink('.filter-change'); //更新許可

    summaryTableFilter.enableFilterDate2('#exec-filter-date2', '#reset-filter-date2', 'period'); //有効期間検索
    summaryTableFilter.enableResetAllFilters('#filter-clear'); //全リセット

    //新規登録
    /*
    const setSearchUsersPlaceholder = () => {
        const val = $('#select-search-user-item').val();
        let item = '';
        if( val === 'login' ){
            item = 'ユーザ名';
        }else if(val === 'email'){
            item = 'メールアドレス';
        }else if(val === 'name'){
            item = '氏名';
        }
        $('#create-new-apikey-search-user-field').attr('placeholder', item ? item + 'を入力' : '');    
    };*/
    const newApikeyNameElm = $('#create-new-apikey-name-field');
    const searchedUsersArea = $('#create-new-apikey-searched-users');
    const searchUsersInput = $('#create-new-apikey-search-user-field');
    const newApikeyUserIdElm = $('#create-new-apikey-userid');

    const toggleExecButton = () => {
        let isPass = true;
        if( !newApikeyNameElm.val() ){
            isPass = false;
        }else if( searchUsersInput.length && !searchUsersInput.val() ){
            isPass = false;
        }else if( !newApikeyUserIdElm.val() ){
            isPass = false;
        }
        $('#exec-regist-new-apikey').prop('disabled', !isPass);
    };

    const resetSearchUsers = () => {
        searchedUsersArea.find('[data-toggle="tooltip"]').tooltip('hide');
        searchedUsersArea.html('');
        newApikeyUserIdElm.val('');
    };

    $('#create-new-apikey').on('click', function(){
        if( searchUsersInput.length ){
            searchUsersInput.val('');
            resetSearchUsers();
        }
        newApikeyNameElm.val('');
        toggleExecButton();
        $('#modalCreateNewApikey').modal('show');
    });

    newApikeyNameElm.on('input', function(){
        toggleExecButton();
    });

    const bindSearchedUserEvent = (elm) => {
        elm.tooltip();
        elm.on('click', function(){
            resetSearchUsers();
            searchUsersInput.val($(this).attr('data-login'));
            newApikeyUserIdElm.val($(this).attr('data-userid'));
            toggleExecButton();
        });
    };

    searchUsersInput.on('input', function(){
        const str = $(this).val();
        if( str ){
            $.post(searchUsersPath, { str }, function(users){
                resetSearchUsers();
                if(users.length && searchUsersInput.val()){
                    searchedUsersArea.append(createSearchedUserElms(str, users));
                    bindSearchedUserEvent(searchedUsersArea.find('[data-userid]'));
                    toggleExecButton();
                }
            });
        }else{
            resetSearchUsers();
        }
    });

    //登録実行
    $('#exec-regist-new-apikey').on('click', function(){
        const button = $(this);
        button.prop('disabled', true);
        const name = newApikeyNameElm.val();
        if( !name ){
            alert('APIキー名を入力して下さい。');
            button.prop('disabled', false);
            return;
        }
        if( searchUsersInput.length ){
            if( !newApikeyUserIdElm.val() ){
                alert('ユーザが選択されておりません。\nユーザを検索後、表示されたユーザから選択して下さい。');
                button.prop('disabled', false);
                return;
            }
        }
        dynamicSummaryTable.executeProcess({
            paramsData: { name, user_id: newApikeyUserIdElm.val() },
            path: $('#apikey-create-path').val(),
            additionalAlertMessage: '登録に失敗しました。'
        });
    });

    //クリップボードにコピー
    const clipboard = new ClipboardJS('.copy-apikey');

    //クリップボードがサポートされていなければアラートを表示 
    $(document).on('click', '.copy-apikry', function(e){
        e.stopPropagation();
        $('#tooltip-copy-btn').tooltip('hide');
        if(clipboard_supported === false) {
            alert('このブラウザではコピー機能はサポートされておりません');
            return false;
        }
    });

    clipboard.on('success', function(e){
        let trigger = $(e.trigger);
        trigger.tooltip('show');
        setTimeout(function (){
            trigger.tooltip('hide');
            e.clearSelection();
        }, 1000);
    });

    clipboard.on('error', function(e) {
        alert('コピーに失敗しました');
        e.clearSelection();
    });

    //一括操作
    const createBulkTrElm = (name) => {
        let tr = $('<tr>');
        tr.append(
            $('<td>').attr('class', 'text-left text-break').text(name)
        );
        return tr;
    };

    //一括削除
    $('#bulk-delete-button').on('click', function(){
        let checked = JSON.parse($('#pages_checked').val());
        $('#bulk-delete-ids').val(JSON.stringify(checked));
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '削除に失敗しました。'
            });
            if( roleOrCategoryError ) return;
            $.post( $('#api-get-apikey-names-path').val(), { ids: JSON.stringify(checked) }, function(names){
                $('#modalBulkDeleteTbody *').remove();
                const sliceNum = 20;
                let nameElms = [];
                const target = $('#modalBulkDeleteTbody');
                names.slice(0, sliceNum).forEach( name => nameElms.push(createBulkTrElm(name)) );
                target.append(nameElms);
                $('#modalBulkDelete').modal('show');
                setTimeout(function(){
                    names.slice(sliceNum).forEach( name => target.append(createBulkTrElm(name)) );
                });
            });
        })();
    });

    $('#exec-bulk-delete').on('click', function(){
        let conf = confirm('一括削除します。よろしいですか?');
        if( !conf ) return;
        dynamicSummaryTable.executeProcessWithProgressBar({
            ids: JSON.parse($('#bulk-delete-ids').val()),
            paramsData: {},
            idsParamsName: 'ids',
            path: $('#apikey-bulk-delete-path').val(),
        });
        return false;
    });
});
