#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmUsersControllerSupport
  include SidfmVmApiControllerSupport
  # 既存のメソッドとメソッド名が被らないように注意

  # ソート
  # orderはascかdescのみを指定する
  def sort_users_content(content:, order:, sidfm_roles:)
    last_index = order.rindex('_')
    column = order.slice(0..last_index - 1)
    order = order.slice(last_index + 1..order.size - 1)

    case column
    when 'login' then
      content = content.order(%|LOWER(login COLLATE "C") #{order.upcase}|)
    when 'lastname' then
      content = content.order(%|LOWER(lastname COLLATE "C") #{order.upcase}|)
                       .order(%|LOWER(login COLLATE "C") #{order.upcase}|)
    when 'mail_notification' then
      content = content.order(%|
        CASE 
        WHEN users.mail_notification = 'only_my_events' THEN
          #{order == 'asc' ? '0' : '10'}
        ELSE 
          #{order == 'asc' ? '10' : '0'}
        END
      |).order('LOWER(login COLLATE "C") ASC')
    when 'lock' then
      if sidfm_roles.include?(:admin) then
        content = content.order(%|
          CASE 
          WHEN status = #{User::STATUS_LOCKED} THEN
            #{order == 'asc' ? '0' : '10'}
          ELSE
            #{order == 'asc' ? '10' : '0'}
          END
        |).order(%|LOWER(login COLLATE "C") #{order.upcase}|)
      end
    when 'last_login_on' then
      content = content.order(%|
        last_login_on #{order.downcase == 'asc' ? 'DESC NULLS LAST' : 'ASC NULLS FIRST'}
      |)
    else
      content = content.order(column => order.to_sym).order(%|LOWER(login COLLATE "C") #{order.upcase}|)
    end
    content
  end

  # フィルタ適用
  def filter_users_content(content:, filters:, sidfm_project_id:, current_user:, sidfm_roles:)

    if filters[:login] && filters[:login].length > 0 then
      if filters[:login] != current_user.login then
        content = content.where('login ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:login])}%")
      else
        content = content.where(:login => filters[:login])
      end
    end

    if filters[:name] && filters[:name].length > 0 then
      content = content.where(
        'firstname ILIKE ? OR lastname ILIKE ?',
        "%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:name])}%",
        "%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:name])}%"
      )
    end

    if filters[:auth] && filters[:auth].length > 0 then
      auth = filters[:auth].to_i
      if auth == -1 then
        content = content.where(auth_source_id: nil)
      else
        content = content.joins("LEFT OUTER JOIN auth_sources ON users.auth_source_id = auth_sources.id")
                         .where("auth_sources.id = ?", auth)
      end
    end

    if filters[:role] && filters[:role].length > 0 then
      role_id = filters[:role].to_i
      #if auth == -1 then
      #  content = content.where.not(:admin => true).where.not(:id => Member.all.select(:user_id))
      #else
        content = content.where(
          :id => content.joins("
            LEFT OUTER JOIN members ON project_id = #{sidfm_project_id} AND members.user_id=users.id
          ")
          .joins("LEFT OUTER JOIN member_roles ON members.id = member_roles.member_id")
          .joins("LEFT OUTER JOIN roles ON member_roles.role_id = roles.id")
          .where("roles.id = ?", role_id)
          .select(:id)
        )
      #end
    end

    if filters[:category] && filters[:category].to_i != 0 then
      category = filters[:category].to_i
      if !sidfm_roles.include?(:admin) && !current_user.categories.where(:id => category).exists? then
        category = 0
      end
      content = content.where(
        :id => content.joins("
          LEFT OUTER JOIN sidfmd_categories_users ON sidfmd_categories_users.user_id = users.id
        ")
        .joins("
          LEFT OUTER JOIN sidfmd_categories ON sidfmd_categories_users.sidfmd_category_id = sidfmd_categories.id"
        )
        .where("sidfmd_categories.id = ?", category)
        .select(:id)
      )
    end

    if sidfm_roles.include?(:user_admin) &&
      filters[:group] &&
      filters[:group].length > 0 &&
      filters[:group].to_i != 0
    then
      if filters[:group].to_i == -1 then
        content = content.where.not(
          :id => SidfmdCatGroupsUser.select(:user_id)
        )
      else
        content = content.where(
          :id => SidfmdCatGroupsUser.where(:sidfmd_cat_group_id => filters[:group].to_i).select(:user_id)
        )
      end
    end

    if !filters[:mail_notification].nil? && filters[:mail_notification].length > 0 then
      if filters[:mail_notification].to_s == "true" then
        content = content.where(:mail_notification => "only_my_events")
      elsif filters[:mail_notification].to_s == "false" then
        content = content.where(:mail_notification => "none")
      end
    end

    if filters[:last_login_on] && filters[:last_login_on].size > 0 then
      begin_date = filters[:last_login_on][0].to_s
      end_date = filters[:last_login_on][1].to_s
      begin
        if begin_date == 'not-login-users' then
          content = content.where('last_login_on IS NULL')
        elsif begin_date.size > 0 && end_date.size > 0 then
          begin_date = Date.parse(begin_date)
          end_date = Date.parse(end_date)
          content = content.where('last_login_on >= ? AND last_login_on < ?', begin_date, end_date + 1.days)
        elsif begin_date.size > 0 then
          begin_date = Date.parse(begin_date)
          content = content.where('last_login_on >= ?', begin_date)
        elsif  end_date.size > 0 then
          end_date = Date.parse(end_date)
          content = content.where('last_login_on < ?', end_date + 1.days)
        end
      rescue ArgumentError => e
        log_exception(e)
        redirect_to :action => :index, :notice => '検索した日付に誤りがあります。' and return
      end
    end

    if sidfm_roles.include?(:admin) && filters[:locked] && filters[:locked].to_s.size > 0 && sidfm_roles.include?(:admin) then
      if filters[:locked].to_s == 'true' then
        content = content.where(:status => User::STATUS_LOCKED)
      elsif filters[:locked].to_s == 'false' then
        content = content.where.not(:status => User::STATUS_LOCKED)
      end
    end

    content
  end

  # 新規登録
  def create_user(
    login:,
    password:,
    lastname:,
    firstname:,
    email:,
    auth_id:,
    role_id:,
    mail_notification:,
    must_change_password:,
    user_lock:,
    category_ids:,
    cat_group_ids:,
    last_login_on: '',
    sidfm_roles:,
    sidfm_project_id:,
    avaliable_role_names:
  )

    category_ids = category_ids.map(&:to_i)
    cat_group_ids = cat_group_ids.map(&:to_i)
    unless sidfm_roles.include?(:admin) then
      category_ids = category_ids & User.current.categories.pluck(:id)
      cat_group_ids = cat_group_ids & User.current.sidfmd_cat_groups.pluck(:id)
      if category_ids.size == 0 && cat_group_ids.size == 0 then
        return create_process_result "#{User::ROLE_SIDFM_ADM}以外の権限には、カテゴリまたはカテゴリグループの指定が少なくとも1つは必要です。"
      end
    end

    begin
      User.transaction do
        unless @sidfm_roles.include?(:admin) then
          if Role.where(:id => role_id).where(:name => User::ROLE_SIDFM_ADM).exists? then
            raise Exception #不正
          end
        end
        if !Role.where(:id => role_id).where(:name => avaliable_role_names).exists? then
          return create_process_result '無効な権限です。'
        end
        mail_notification = if mail_notification.nil? || mail_notification.to_s == "false" then
                              "none"
                            elsif mail_notification.to_s == "true" then
                              "only_my_events"
                            else
                              nil
                            end
        unless mail_notification then
          return create_process_result 'メール通知設定の指定に誤りがあります。'
        end

        must_change_password = if must_change_password.nil? || must_change_password.to_s == "false" then
                                  false
                                elsif must_change_password.to_s == "true" then
                                  true
                                end
 
        if auth_id == -2 then
          return create_process_result '無効な認証方式です。'
        end

        # VM APIの時だけlast_login_onは指定可能(nilでない可能性がある)
        if last_login_on && sidfm_roles.include?(:user_admin) then
          if last_login_on.size == 0 then
            last_login_on == nil
          else
            begin
              last_login_on = DateTime.parse last_login_on
            rescue ArgumentError => e
              log_exception(e)
              return create_process_result '指定した日時が正しくありません。'
            end
          end
        end

        user = User.new
        user.login = login
        user.password = password
        user.password_confirmation = password
        user.lastname = lastname
        user.firstname = firstname
        user.mail = email
        user.mail_notification = mail_notification
        user.must_change_passwd = must_change_password if sidfm_roles.include?(:user_admin)
        user.status = User::STATUS_LOCKED if user_lock.to_s == 'true' && sidfm_roles.include?(:admin)
        user.last_login_on = last_login_on if sidfm_roles.include?(:user_admin) && !last_login_on.is_a?(String)

        user.sidfmd_categories = SidfmdCategory.where(:id => category_ids) if category_ids.size > 0
        if cat_group_ids.size > 0 && !Role.where(:id => role_id).where(:name => User::ROLE_SIDFM_ADM).exists? then
          # 管理者権限はグループ付与不可
          user.sidfmd_cat_groups = SidfmdCatGroup.where(:id => cat_group_ids)
        end

        unless auth_id == -1 then
          user.auth_source_id = auth_id
        end

        if user.save then
          #ユーザ権限の登録
          newmem = Member.create_principal_memberships(user, :project_ids => [ sidfm_project_id ], :role_ids => role_id)
          if !newmem then
            logger.info 'failed to create member'
            raise Exception
          end
        else
          return create_process_result user.errors.full_messages.join('。') + "。"
        end
        return create_process_result nil, { :id => user.id }
      end
    rescue Exception => e
      log_exception(e)
      return create_process_result 'ユーザの登録に失敗しました。'
    end
  end


  # 変更
  def update_user(
    user_id:,
    login:,
    password:,
    lastname:,
    firstname:,
    email:,
    auth_id:,
    role_id:,
    mail_notification:,
    must_change_password:,
    user_lock:,
    last_login_on: '',
    sidfm_roles:,
    sidfm_project_id:,
    avaliable_role_names:,
    category_ids: nil,
    cat_group_ids: nil
  )
    begin
      User.transaction do
        user = User.lock.find(user_id.to_i)
        if !sidfm_roles.include?(:user_admin) and user.id != User.current.id then
          return create_process_result '指定したユーザの操作は許可されていません。'
        end

        role_id = role_id.to_i
        auth_id = auth_id.to_i
        if !Role.where(:id => role_id).where(:name => avaliable_role_names).exists? then
          return create_process_result '無効な権限です。'
        end
        mail_notification = if mail_notification.nil? || mail_notification.to_s == "false" then
                              "none"
                            elsif mail_notification.to_s == "true" then
                              "only_my_events"
                            else
                              nil
                            end 
        unless mail_notification then
          return create_process_result 'メール通知設定の指定に誤りがあります。'
        end
        must_change_password = if must_change_password.nil? || must_change_password.to_s == "false" then
                                 false
                               elsif must_change_password.to_s == "true" then
                                 true
                               end

        if auth_id == -2 then
          return create_process_result '無効な認証方式です。'
        end

        unless sidfm_roles.include?(:user_admin) then
          sidfm_view_id = Role.find_by(name: User::ROLE_SIDFM_USR).id
          sidfm_ope_id = Role.find_by(name: User::ROLE_SIDFM_USE).id
          user_authsource_id = User.current.auth_source_id.to_i
          user_authsource_id = -1 if user_authsource_id == 0
          unless user == User.current && auth_id == user_authsource_id && (role_id == sidfm_view_id or role_id == sidfm_ope_id) then
            return create_process_result '指定した操作は許可されていません。'
          end
        end
        if user.login == 'sidfmadm' && login != user.login then
          return create_process_result '指定したユーザのログイン名の更新は許可されていません。'
        end
        if sidfm_roles.include?(:user_admin) then
          if auth_id == -1 and (!user.hashed_password or user.hashed_password.size == 0) and (!password or password.size == 0) then
            return create_process_result '認証方式が「パスワード認証」の場合は、パスワードが必須です。'
          end
        end

        # VM APIの時だけlast_login_onは指定可能
        if last_login_on.is_a?(String) && sidfm_roles.include?(:user_admin) then
          if last_login_on.size == 0 then
            last_login_on = nil
          else
            begin
              last_login_on = DateTime.parse last_login_on
            rescue ArgumentError => e
              log_exception(e)
              return create_process_result '指定した日時が正しくありません。'
            end
          end
        end

        admin_ids = Role.where(:name => User::ROLE_SIDFM_ADM).pluck(:id)
        admin_users = User.joins(
          "LEFT OUTER JOIN members ON members.project_id = #{sidfm_project_id} AND members.user_id = users.id"
        ).joins(
          "LEFT OUTER JOIN member_roles ON member_roles.member_id = members.id"
        ).where("member_roles.role_id IN (?)", admin_ids)

        if (!admin_ids.include?(role_id) && admin_users.include?(user) && admin_users.size == 1) ||
           (user.login == 'sidfmadm' && !admin_ids.include?(role_id))
        then
          return create_process_result '指定したユーザの権限は変更できません。'
        end
 
        # VM APIの時だけcategory_idsとcat_group_idsは指定可能(nilでない可能性がある)
        if !sidfm_roles.include?(:user_admin) && (category_ids || cat_group_ids) then
          return create_process_result 'カテゴリ及びカテゴリグループの更新は許可されておりません。'
        end
        if category_ids then
          category_ids = category_ids.map(&:to_i)
          unless sidfm_roles.include?(:admin) then
            unchangeable_category_ids = user.sidfmd_categories.where.not(:id => User.current.categories.select(:id)).pluck(:id)
            category_ids = (category_ids + unchangeable_category_ids).uniq
          end
          user.sidfmd_categories = SidfmdCategory.where(:id => category_ids)
          #関連するルールの削除・変更
          process_rules = User.process_related_rules(
            :sidfm_roles => sidfm_roles,
            :change_category_user_ids => { user.id.to_s => (category_ids + user.cat_group_categories.pluck(:id)).uniq },
            :action => 'category_change'
          )
          if !process_rules then
            logger.error('failed to process rules')
            raise Exception
          end
        end

        if cat_group_ids then
          cat_group_ids = cat_group_ids.map(&:to_i)
          if !admin_ids.include?(role_id) && (sidfm_roles.include?(:admin) || user != User.current) then
            user.update_cat_groups(
              :current_user_id => User.current.id,
              :role_admin => sidfm_roles.include?(:admin),
              :cat_group_ids => cat_group_ids
            )
          end
          #関連するルールの削除・変更
          user_categoryids = { #グループ変更後のユーザ毎のカテゴリ
            user.id.to_s => User.after_cat_group_change_categories(
              :user_id => user.id,
              :cat_group_ids => cat_group_ids
            )
          }
          unless sidfm_roles.include?(:admin) then #ユーザに設定されていないカテゴリグループのカテゴリを入れる
            user_categoryids[user.id.to_s] |= SidfmdCatGroup.limit_user_group_category_ids(user.id, User.current.id)
          end

          process_rules = User.process_related_rules(
            :sidfm_roles => sidfm_roles,
            :change_category_user_ids => user_categoryids,
            :action => 'bulk_category_change'
          )
          if !process_rules then
            logger.error('failed to process rules')
            raise Exception
          end
        end

        if sidfm_roles.include?(:user_admin) then
          user.update(:login => login) if login != 'sidfmadm'
          user.update(:last_login_on => last_login_on) unless last_login_on.is_a?(String)
        end
        user.update(
          :lastname => lastname,
          :firstname => firstname
        )
        user.update(:mail => email)
        if sidfm_roles.include?(:user_admin) or !Sidfm.restrict_mail_notification_setting then
          user.update(:mail_notification => mail_notification)
        end
        user.update(:must_change_passwd => must_change_password) if sidfm_roles.include?(:user_admin)

        is_delete_session_necessary = false
        if sidfm_roles.include?(:admin) && user_lock.to_s == 'true' && user.id != User.current.id then
          user.update(:status => User::STATUS_LOCKED)
          is_delete_session_necessary = false
        elsif sidfm_roles.include?(:admin) && (!user_lock || user_lock.to_s == 'false') then
          user.update(:status => User::STATUS_ACTIVE)
        end
        if password && password.size > 0 then
          user.update_attribute(:password, password)
          user.update_attribute(:password_confirmation, password)
          is_delete_session_necessary = true
        end
        if sidfm_roles.include?(:user_admin) then
          if user.auth_source_id.to_i != auth_id then
            if auth_id <= 0 then
              user.update(:auth_source_id => nil)
            else
              user.update(:auth_source_id => auth_id)
            end
            is_delete_session_necessary = true
          end
        end
        member_role_id = 0
        if sidfm_roles.include?(:user_admin) then
          role_before_update = user.roles.where(:members => { :project_id => sidfm_project_id }).first
          role_id_before_update = role_before_update ? role_before_update.id : nil
          if user.admin == false && Role.where(:name => '管理者').where(:id => role_id).exists? then
            logger.info('invalid role_id')
            raise Exception
          else
            member_role = MemberRole.joins(:member)
              .where(:members => { :project_id => sidfm_project_id, :user_id => user_id })
              .first
            member_role_id = member_role ? member_role.id : -2
            if member_role_id != -2 then
              if role_before_update.to_s != role_id.to_s then
                #関連するルールの処理
                process_rules = User.process_related_rules(
                  :sidfm_roles => sidfm_roles,
                  :change_role_user_ids => { user.id.to_s => role_id.to_i },
                  :action => 'auth_change'
                )
                if !process_rules then
                  logger.error('failed to process rules')
                  raise Exception
                end
                if admin_ids.include?(role_id) then
                  #管理者に変更ならカテゴリグループの紐付けはなしにする
                  user.sidfmd_cat_groups = []
                end
                if Role.where(:id => role_id, :name => User::ROLE_SIDFM_USR).exists? then
                  #閲覧者に変更なら、VmApikeyをチェックして更新許可がtrueをfalseに変更
                  SidfmdVmApikey.update_user_apikeys_allow_update user.id
                end
              end
              unless MemberRole.find(member_role_id).update_attribute :role_id, role_id then
                logger.error('failed to update role')
                raise Exception
              end
            end
          end
        end

        if sidfm_roles.include?(:user_admin) then
            #if role_id == -1 then
            #  if member_del = Member.where(:project_id => @sidfm_project.id).joins(:user).where("users.id = ?", checked[:p_id]).first
            #    #関連するルールの処理
            #    process_rules = User.process_related_rules(
            #      :change_role_user_ids => { user.id.to_s => role_id.to_i },
            #      :action => 'auth_change'
            #    )
            #    if !process_rules then
            #      logger.error('failed to process rules')
            #      raise Exception
            #    end
            #    unless member_del.destroy then
            #      logger.error('failed to delete role')
            #      raise Exception
            #    end
            #  end 
          if member_role_id == -2 then
            newmem = Member.create_principal_memberships(user, :project_ids => [ sidfm_project_id ], :role_ids => role_id)
            unless newmem then
              logger.info 'failed to create member'
              raise Exception
            end
          end
        end

        if !category_ids.nil? then
          user.sidfmd_categories = SidfmdCategory.where(:id => category_ids)
        end
        if user.id == User.current.id
          # The session token was destroyed by the password change, generate a new one
          session[:tk] = user.generate_session_token
        else
          Session.delete_session(user.id) if is_delete_session_necessary
        end
      end
      Rails.cache.clear
    rescue Exception => e
      log_exception(e)
      return create_process_result 'ユーザ情報の更新に失敗しました。'
    end
    return create_process_result
  end

  # 削除
  def delete_user(user_id:, sidfm_roles:)
    user_id = user_id.to_i
    begin
      User.transaction do
        user = User.lock.find(user_id)
        if user.admin == true ||
          user == User.current ||
          user.login == 'sidfmadm'
        then
          return create_process_result '指定したユーザは削除できません。'
        end
        process_rules = User.process_related_rules(
          :sidfm_roles => sidfm_roles,
          :delete_user_ids => [ user.id ],
          :action => 'delete'
        )
        if !process_rules then
          logger.info('failed to process rules')
          raise Exception
        end
        if user.destroy then
          Session.delete_session(user_id)
        end
      end
    rescue Exception => e
      log_exception(e)
      return create_process_result 'ユーザの削除に失敗しました。'
    end
    Rails.cache.clear
    return create_process_result
  end

  def get_openid_connect_auth_id
    auth = AuthSource.find_by(:name => User::OPENID_CONNECT_AUTH_NAME)
    auth_id = auth ? auth.id : -10
  end

  def validate_fetch_params(method, filtered_params)
    unless filtered_params.is_a?(Hash)
      return nil
    end
    filtered_params = filtered_params.symbolize_keys
    p_id = filtered_params[:userid].to_i
    p_name = filtered_params[:login]
    p_pw = filtered_params[:pw]
    p_pwconf = filtered_params[:pwconf]
    p_lastname = filtered_params[:lastname]
    p_firstname = filtered_params[:firstname]
    p_email = filtered_params[:email]
    p_auth = filtered_params[:auth].to_i
    func = method
    openid_connect_auth_id = get_openid_connect_auth_id
    if !login_valid?(p_name) then
      return { :sccess => false, :notice => 'ログイン名に使用できない文字が含まれています。'}
    end
    if func == 'cre' && (p_pw.nil? or p_pw.size == 0) && openid_connect_auth_id != p_auth then
      return { :success => false, :notice => 'パスワードが入力されていません。' }
    end
    if func == 'cre' && (p_pwconf.nil? or p_pwconf.size == 0) && openid_connect_auth_id != p_auth then
      return { :success => false, :notice => 'パスワード確認を入力して下さい。'}
    elsif p_pwconf && p_pw && p_pwconf != p_pw then
      return { :success => false, :notice => 'パスワードが一致しません。'}
    end
    if p_name.nil? or p_name.size == 0 then
      return { :success => false,  :notice => 'ログイン名が入力されていません。'}
    end
    if p_lastname.nil? or p_lastname.size == 0 then
      return { :success => false,  :notice => '名字が入力されていません。'}
    end
    if p_firstname.nil? or p_firstname.size == 0 then
      return { :success => false, :notice => '名前が入力されていません。'}
    end

    if p_name.size > User::LOGIN_LENGTH_LIMIT then
      return { :success => false, :notice => "ログイン名は#{User::LOGIN_LENGTH_LIMIT}文字以内で入力してください。" }
    end

    if !mail_address_valid?(p_email) then
      return { :success => false, :notice => 'メールアドレスに誤りがあります。'}
    end

    if p_email.size > User::MAIL_LENGTH_LIMIT then
      return { :success => false, :notice => "メールアドレスは#{User::MAIL_LENGTH_LIMIT}文字以内で入力してください。" }
    end

    if (func == 'cre' && openid_connect_auth_id != p_auth) || (p_pw && p_pw.size > 0) then
      if p_pw.size < Setting.password_min_length.to_i then
        return { :success => false,  :notice => 'パスワードは' + Setting.password_min_length + '文字以上で入力してください。'}
      end
    end

    if p_pw && p_pw.size > 0 then
      temp_user = User.new
      temp_user.password = p_pw
      temp_user.login = p_name
      is_pw_valid = temp_user.check_password_with_password_policy
      if !is_pw_valid then
        return { :success => false,  :notice => 'パスワードが、パスワードポリシーに違反しています。' }
      end
    end

    if p_lastname.size > User::LASTNAME_LENGTH_LIMIT then
      return { :success => false, :notice => "姓は#{User::LASTNAME_LENGTH_LIMIT}文字以内で入力してください。" }
    end
    if p_firstname.size > User::FIRSTNAME_LENGTH_LIMIT then
      return { :success => false, :notice => "名は#{User::FIRSTNAME_LENGTH_LIMIT}文字以内で入力してください。" }
    end
    if func == 'mod' then
      same_login_user = User.find_by(login: p_name)
      if same_login_user then
        unless same_login_user.id == p_id then
          return { :success => false, :notice => 'ログイン名は既に使用されています。'}
        end
      end
    else
      if User.find_by(login: p_name) then
        return { :success => false, :notice => 'ログイン名は既に使用されています。'}
      end
    end

    { :success => true, :p_id => p_id, :p_name => p_name, :p_pw => p_pw, :p_pwconf => p_pwconf, :p_lastname => p_lastname, :p_firstname => p_firstname, :p_email => p_email}
  end

  def login_valid?(login)
    login.match(/\A[a-z0-9_\-@\.]*\z/i)
  end

  def mail_address_valid?(mail_address)
    mail_address.match(/\A([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})\z/i) and mail_address.match(/\A[a-zA-Z0-9_\#!$%&`'*+\-{|}~^\/=?\.]+@[a-zA-Z0-9][a-zA-Z0-9\.-]+\z/)
  end

end
