#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdIssueCve < ActiveRecord::Base
  belongs_to :sidfmd_issue

  scope :enable_hosts, -> { joins(:sidfmd_issue).joins(:sidfmd_issue => [ :sidfmd_host ]).where(:sidfmd_hosts => { :flg_delete => false }) }

  scope :enable_categories, -> { joins(:sidfmd_issue).where(:sidfmd_issues => { :sidfmd_host_id => SidfmdCategoriesHost.select(:sidfmd_host_id).distinct(:sidfmd_host_id)}) }

  scope :exploitable, -> { joins(:sidfmd_issue).where('c2_exploitable IS TRUE OR c3_exploitable IS TRUE').where(:sidfmd_issues => { :sidfmd_host_id => SidfmdHost.enable_hosts.select(:id) }).group(:sidfmd_host_id) }

  NOT_REGISTERED = '未登録'.freeze
  RISK_FIXED = '済'.freeze


  # cvss{:label, :score}から出力用の文字列を返す。
  def self.display_cvss(i)
    case i[:label]
    when :notreg then NOT_REGISTERED
    else sprintf('%2.1f', i[:score])
    end
  end

  LABEL = {
    :notreg => nil,
    :d4 => { :top => 10.0 , :bottom => 9.0 }.freeze,
    :d3 => { :top => 9.0, :bottom => 7.0 }.freeze,
    :d2 => { :top => 7.0, :bottom => 4.0 }.freeze,
    :d1 => { :top => 4.0, :bottom => 0.0 }.freeze,
  }.freeze

  LABELS = [ :notreg, :d1, :d2, :d3, :d4 ].map(&:freeze).freeze
  DISPLAY_SORTED_LABELS = [ :notreg, :d4, :d3, :d2, :d1 ].map(&:freeze).freeze

  def self.label2value(label)
    idx = LABELS.index(label)
    idx ? idx : 0
  end

  # cvssスコアからラベルを取得する。
  def self.labelling(s)
    if s.nil?                                               then :notreg
    elsif                          s >= LABEL[:d4][:bottom] then :d4
    elsif s < LABEL[:d3][:top] and s >= LABEL[:d3][:bottom] then :d3
    elsif s < LABEL[:d2][:top] and s >= LABEL[:d2][:bottom] then :d2
    else                                                         :d1
    end
  end

  #cvss v2の項目と選択肢を返す
  def self.get_cvss_v2_vectors
    vectors = {
      'AV' => {
        'item_name' => '攻撃元の場所&nbsp;(AV)',
        'values' => {
          'L' => 'ローカル&nbsp;(L)',
          'A' => '隣接&nbsp;(A)',
          'N' => 'ネットワーク&nbsp;(N)',
        }
      },
      'AC' => {
        'item_name' => '攻撃成立条件の難易度&nbsp;(AC)',
        'values' => {
          'H' => '難しい&nbsp;(H)',
          'M' => 'やや難しい&nbsp;(M)',
          'L' => '簡単&nbsp;(L)'
        }
       },
       'Au'=> {
         'item_name' => '攻撃前の認証の要否&nbsp;(Au)',
         'values' => {
           'M' => '複数回&nbsp;(M)',
           'S' => '一回&nbsp;(S)',
           'N' => '不要&nbsp;(N)'
         }
       }, 
       'C' => { 
         'item_name' => '情報漏洩への影響&nbsp;(C)',
         'values' => {
           'N' => 'なし&nbsp;(N)',
           'P' => '部分的にあり&nbsp;(P)',
           'C' => '全面的にあり&nbsp;(C)'
         }
       },
       'I' => {
         'item_name' => '情報改竄への影響&nbsp;(I)',
         'values' => {
           'N' => 'なし&nbsp;(N)',
           'P' => '部分的にあり&nbsp;(P)',
           'C' => '全面的にあり&nbsp;(C)'
         }
       },
       'A' => {
         'item_name' => '業務妨害への影響&nbsp;(A)',
         'values' => {
           'N' => 'なし&nbsp;(N)',
           'P' => '部分的にあり&nbsp;(P)',
           'C' => '全面的にあり&nbsp;(C)'
         }
       }
     }
  end

  #cvss v3の項目と選択肢を返す
  def self.get_cvss_v3_vectors
    vectors = {
      'AV' => {
        'item_name' => '攻撃元区分&nbsp;(AV)',
        'values' => {
          'P' => '物理&nbsp;(P)',
          'L' => 'ローカル&nbsp;(L)',
          'A' => '隣接&nbsp;(A)',
          'N' => 'ネットワーク&nbsp;(N)',
        }
      },
      'AC' => {
        'item_name' => '攻撃条件の複雑さ&nbsp;(AC)',
        'values' => {
          'H' => '高&nbsp;(H)',
          'L' => '低&nbsp;(L)',
        }
      },
      'PR' => {
        'item_name' => '必要な特権レベル&nbsp;(PR)',
        'values' => {
          'H' => '高&nbsp;(H)',
          'L' => '低&nbsp;(L)',
          'N' => '不要&nbsp;(N)',
        }
      },
      'UI' => {
        'item_name' => 'ユーザ関与レベル&nbsp;(UI)',
        'values' => {
          'R' => '要&nbsp;(R)',
          'N' => '不要&nbsp;(N)',
        }
      },
      'S' => {
        'item_name' => 'スコープ&nbsp;(S)',
        'values' => {
          'U' => '変更なし&nbsp;(U)',
          'C' => '変更あり&nbsp;(C)',
        },
      },
      'C' => {
        'item_name' => '情報漏洩の影響&nbsp;(C)',
        'values' => {
          'N' => 'なし&nbsp;(N)',
          'L' => '低&nbsp;(L)',
          'H' => '高&nbsp;(H)',
        }
      },
      'I' => {
        'item_name' => '情報改竄の影響&nbsp;(I)',
        'values' => {
          'N' => 'なし&nbsp;(N)',
          'L' => '低&nbsp;(L)',
          'H' => '高&nbsp;(H)',
        }
      },
      'A' => {
        'item_name' => '業務妨害の影響&nbsp;(A)',
        'values' => {
          'N' => 'なし&nbsp;(N)',
          'L' => '低&nbsp;(L)',
          'H' => '高&nbsp;(H)',
        }
      }
    }
  end

end
